// Poor Data Injector
// 2015-2016 Seyma Nur Soydemir seyma@student.dei.uc.pt

package intercept;

import java.math.BigDecimal;
import java.sql.Date;
import java.sql.Ref;
import java.util.ArrayList;
import java.util.Random;

import mutation.Operators_Binary;
import mutation.Operators_Boolean;
import mutation.Operators_Date;
import mutation.Operators_Decimal;
import mutation.Operators_Double;
import mutation.Operators_Integer;
import mutation.Operators_Object;
import mutation.Operators_References;
import mutation.Operators_String;
import mutation.Operators_Time;
import mutation.Operators_Timestamp;

public class MutationPerformer {
 // Amount of different data groups that we have mutations for
 private static final int number_of_types = 11;

 // Counters, each position is attributed to a group
 public static int counter[] = {0,0,0,0,0,0,0,0,0,0,0};
 public static int param1counter[] = {0,0,0,0,0,0,0,0,0,0,0};
 public static int param2counter[] = {0,0,0,0,0,0,0,0,0,0,0};

 private static Object lock = new Object();

 // Performs the mutations for Strings, adapting to the type of mutation (and parameters that it takes) as needed
 public static Object performString(String resultString) throws Throwable {
  int counter_id = 0;

  // Add the mutations to be done in their correct order
  ArrayList < Object > sequence = new ArrayList < Object > ();

  sequence.add(Operators_String.m1_replaceByNull);
  sequence.add(Operators_String.m2_replaceByEmptyString);
  sequence.add(Operators_String.m3_replaceByNonprintable);
  sequence.add(Operators_String.m5_replaceWithAlphanumeric);
  sequence.add(Operators_String.m15_replaceBySQLString);
  sequence.add(Operators_String.m16_replaceAllBySymbol);
  sequence.add(Operators_String.m17_misspeling);
  sequence.add(Operators_String.m21_replaceWhitespaceBySymbols);
  sequence.add(Operators_String.m4_addRandomNonprintable);
  sequence.add(Operators_String.m6_addRandomNumeric);
  sequence.add(Operators_String.m9_addCharactersToString);
  sequence.add(Operators_String.m7_convertOppositeCase);
  sequence.add(Operators_String.m13_removeWhitespace);
  sequence.add(Operators_String.m22_addExtraneous);
  sequence.add(Operators_String.m8_insertWhitespace);
  sequence.add(Operators_String.m12_removeSubstring);
  sequence.add(Operators_String.m14_delimitedReverse);
  sequence.add(Operators_String.m10_addSubstring);
  sequence.add(Operators_String.m11_moveSubstring);
  sequence.add(Operators_String.m19_replaceByAcronymn);
  sequence.add(Operators_String.m20_replaceBySpecificTypeString);
  sequence.add(Operators_String.m18_replaceLimitSize);
  sequence.add(Operators_String.m23_replaceByHomonyms);
  sequence.add(Operators_String.m24_replaceBySynonyms);


  Object functionToCall = null;
  int size = 0;
  Random random = new Random();
  int randomBegin = 0;
  int randomLength = 0;
  int quantity = 0;
  String temp = null;

  // Gets the next mutation to be done
  synchronized(lock) {
   functionToCall = sequence.get(counter[counter_id]);
  }

  if (functionToCall instanceof Operators_String.functionOneParameter) {
   synchronized(lock) {
    counter[counter_id]++;
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultString = ((Operators_String.functionOneParameter) functionToCall).run(resultString);

  }

  String[] positionParameter1 = {
   "prefix",
   "middle",
   "suffix"
  };
  if (functionToCall instanceof Operators_String.functionTwoParameterPosition) {
   synchronized(lock) {
    temp = positionParameter1[param1counter[counter_id]];

    param1counter[counter_id]++;
    if (param1counter[counter_id] == positionParameter1.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultString = ((Operators_String.functionTwoParameterPosition) functionToCall).run(resultString, temp);

  }
  String[] positionParameter2 = {
   "first character",
   "last character",
   "first character of each word",
   "last character of each word",
   "random position",
   "random position of each word"
  };
  if (functionToCall instanceof Operators_String.functionTwoParameter2) {
   synchronized(lock) {
    temp = positionParameter2[param1counter[counter_id]];
    param1counter[counter_id]++;
    if (param1counter[counter_id] == positionParameter2.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultString = ((Operators_String.functionTwoParameter2) functionToCall).run(resultString, temp);


  }

  String[] positionParameter3 = {
   "leading",
   "trailing",
   "random between words"
  };
  if (functionToCall instanceof Operators_String.functionTwoParameter3) {
   synchronized(lock) {
    temp = positionParameter3[param1counter[counter_id]];

    param1counter[counter_id]++;
    if (param1counter[counter_id] == 3) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultString = ((Operators_String.functionTwoParameter3) functionToCall).run(resultString, temp);


  }

  String[] positionParameter4 = {
   "before",
   "after"
  };
  if (functionToCall instanceof Operators_String.functionTwoParameter4) {
   synchronized(lock) {
    temp = positionParameter4[param1counter[counter_id]];

    param1counter[counter_id]++;
    if (param1counter[counter_id] == positionParameter4.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultString = ((Operators_String.functionTwoParameter4) functionToCall).run(resultString, temp);



  }

  String[] typeParameter2 = {
   "UTF maximum",
   "UTF maximum + 1"
  };

  if (functionToCall instanceof Operators_String.functionTwoParameter5) {
   synchronized(lock) {
    temp = typeParameter2[param1counter[counter_id]];

    param1counter[counter_id]++;
    if (param1counter[counter_id] == typeParameter2.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultString = ((Operators_String.functionTwoParameter5) functionToCall).run(resultString, temp);

  }

  if (functionToCall instanceof Operators_String.functionThreeParameter) {
   synchronized(lock) {
    temp = positionParameter3[param1counter[counter_id]];

    param1counter[counter_id]++;
    if (param1counter[counter_id] == positionParameter3.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   if ((resultString == null) || (resultString.equals(""))) {
    quantity = -1;
   } else {
    quantity = random.nextInt(resultString.length()) + 1;
   }
   resultString = ((Operators_String.functionThreeParameter) functionToCall).run(resultString, temp, quantity);

  }

  if (functionToCall instanceof Operators_String.functionThreeParameter2) {
   synchronized(lock) {
    counter[counter_id]++;
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   if ((resultString == null) || (resultString.equals(""))) {
    size = -1;
    randomBegin = -1;
    randomLength = -1;
   } else {
    size = resultString.length();
    randomBegin = random.nextInt(size);
    randomLength = random.nextInt(size - randomBegin) + 1;
   }
   resultString = ((Operators_String.functionThreeParameter2) functionToCall).run(resultString, randomBegin, randomLength);

  }

  if (functionToCall instanceof Operators_String.functionFourParameter1) {
   synchronized(lock) {
    temp = positionParameter1[param1counter[counter_id]];

    param1counter[counter_id]++;
    if (param1counter[counter_id] == positionParameter1.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   if ((resultString == null) || (resultString.equals(""))) {
    size = -1;
    randomBegin = -1;
    randomLength = -1;
   } else {
    size = resultString.length();
    randomBegin = random.nextInt(size);
    randomLength = random.nextInt(size - randomBegin) + 1;
   }
   resultString = ((Operators_String.functionFourParameter1) functionToCall).run(resultString, randomBegin, randomLength, temp);

  }

  return resultString;

 }

 // Performs the mutations for Long, adapting to the type of mutation (and parameters that it takes) as needed
 public static Object performLong(Long resultLong) throws Throwable {
  int counter_id = 1;

  // Add the mutations to be done in their correct order
  ArrayList < Object > sequence = new ArrayList < Object > ();
  sequence.add(Operators_Integer.m1_replaceByRandomSamesize);
  sequence.add(Operators_Integer.m2_removeOneNumeric);
  sequence.add(Operators_Integer.m3_duplicateOneNumeric);
  sequence.add(Operators_Integer.m4_invertConsecutiveDigits);
  sequence.add(Operators_Integer.m5_addOneRandomNumeric);
  sequence.add(Operators_Integer.m6_flipSign);
  sequence.add(Operators_Integer.m7_replaceWithRandom);
  sequence.add(Operators_Integer.m8_add1);
  sequence.add(Operators_Integer.m9_subtract1);

  Object functionToCall = null;

  synchronized(lock) {
   functionToCall = sequence.get(counter[counter_id]);
  }

  if (functionToCall instanceof Operators_Integer.functionOneParameter) {
   synchronized(lock) {
    counter[counter_id]++;
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }

   resultLong = ((Operators_Integer.functionOneParameter) functionToCall).run(resultLong);

  } else {
   System.out.println("We didnt enter any integer func");
  }

  return resultLong;
 }

 // Performs the mutations for Date, adapting to the type of mutation (and parameters that it takes) as needed
 public static Object performDate(java.sql.Date resultDate) throws Throwable {
  int counter_id = 2;

  // Add the mutations to be done in their correct order
  ArrayList < Object > sequence = new ArrayList < Object > ();

  sequence.add(Operators_Date.m1_replaceByNull);
  sequence.add(Operators_Date.m2_replaceByEmptyDummyDate);
  sequence.add(Operators_Date.m3_replaceByRandomDate);
  sequence.add(Operators_Date.m4_addOneExtra);
  sequence.add(Operators_Date.m5_removeOneDigit);
  sequence.add(Operators_Date.m6_replaceDigitWithRandom);
  sequence.add(Operators_Date.m7_invertCharacters);
  sequence.add(Operators_Date.m8_swapValues);
  sequence.add(Operators_Date.m9_replaceWithDefault);
  sequence.add(Operators_Date.m10_removeDigitsFromYear);
  sequence.add(Operators_Date.m11_add100years);
  sequence.add(Operators_Date.m12_subtract100years);
  sequence.add(Operators_Date.m13_duplicate);

  Object functionToCall = null;
  String temp = null;
  String temp2 = null;

  synchronized(lock) {
   functionToCall = sequence.get(counter[counter_id]);
  }
  if (functionToCall instanceof Operators_Date.functionOneParameter) {
   synchronized(lock) {
    counter[counter_id]++;
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultDate = ((Operators_Date.functionOneParameter) functionToCall).run(resultDate);

  }

  String[] positionParameterDate = {
   "year",
   "month",
   "day"
  };
  if (functionToCall instanceof Operators_Date.functionTwoParameters) {
   synchronized(lock) {
    temp = positionParameterDate[param1counter[counter_id]];

    param1counter[counter_id]++;
    if (param1counter[counter_id] == positionParameterDate.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultDate = ((Operators_Date.functionTwoParameters) functionToCall).run(resultDate, temp);

  }

  String[] positionParameterDate2 = {
   "Before year value",
   "After year value",
   "Before month value",
   "After month value",
   "Before day value",
   "After day value"
  };
  if (functionToCall instanceof Operators_Date.functionTwoParameters2) {
   synchronized(lock) {
    temp = positionParameterDate2[param1counter[counter_id]];

    param1counter[counter_id]++;
    if (param1counter[counter_id] == positionParameterDate2.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultDate = ((Operators_Date.functionTwoParameters2) functionToCall).run(resultDate, temp);

  }


  Date defaultDate = new Date(1);
  if (functionToCall instanceof Operators_Date.functionTwoParameters3) {
   synchronized(lock) {
    counter[counter_id]++;
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultDate = ((Operators_Date.functionTwoParameters3) functionToCall).run(resultDate, defaultDate);

  }

  String[] sourceParameter = {
   "year",
   "month",
   "day"
  };
  String[] targetParameter = {
   "year",
   "month",
   "day"
  };
  if (functionToCall instanceof Operators_Date.functionThreeParameters) {
   synchronized(lock) {
    temp = sourceParameter[param1counter[counter_id]];
    temp2 = targetParameter[param2counter[counter_id]];

    param2counter[counter_id]++;
    if (param1counter[counter_id] == param2counter[counter_id]) {
     // This means we are trying same thing (e.g., year->year)
     param2counter[counter_id]++;
    }
    if (param2counter[counter_id] >= targetParameter.length) {
     param2counter[counter_id] = 0;
     param1counter[counter_id]++;
    }
    if (param1counter[counter_id] >= sourceParameter.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultDate = ((Operators_Date.functionThreeParameters) functionToCall).run(resultDate, temp, temp2);

  }

  return resultDate;
 }

 // Performs the mutations for Time, adapting to the type of mutation (and parameters that it takes) as needed
 public static Object performTime(java.sql.Time resultTime) throws Throwable {
  int counter_id = 3;

  // Add the mutations to be done in their correct order
  ArrayList < Object > sequence = new ArrayList < Object > ();

  sequence.add(Operators_Time.m1_replaceByNull);
  sequence.add(Operators_Time.m2_replaceByEmptyDummyTime);
  sequence.add(Operators_Time.m3_replaceByRandomTime);
  sequence.add(Operators_Time.m4_reverseField);
  sequence.add(Operators_Time.m5_addOneExtra);
  sequence.add(Operators_Time.m6_duplicateOneDigit);
  sequence.add(Operators_Time.m7_removeOneDigit);

  Object functionToCall = null;
  String temp = null;

  synchronized(lock) {
   functionToCall = sequence.get(counter[counter_id]);
  }

  if (functionToCall instanceof Operators_Time.functionOneParamter) {
   synchronized(lock) {
    counter[counter_id]++;
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultTime = ((Operators_Time.functionOneParamter) functionToCall).run(resultTime);

  }

  String[] field = {
   "hour",
   "minute",
   "second"
  };
  if (functionToCall instanceof Operators_Time.functionTwoParameters) {
   synchronized(lock) {
    temp = field[param1counter[counter_id]];

    param1counter[counter_id]++;
    if (param1counter[counter_id] == field.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultTime = ((Operators_Time.functionTwoParameters) functionToCall).run(resultTime, temp);



  }

  String[] positionParameter5 = {
   "before hour",
   "after hour",
   "before minute",
   "after minute",
   "before second",
   "after second"
  };
  if (functionToCall instanceof Operators_Time.functionTwoParameters2) {
   synchronized(lock) {
    temp = positionParameter5[param1counter[counter_id]];
    param1counter[counter_id]++;
    if (param1counter[counter_id] == positionParameter5.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultTime = ((Operators_Time.functionTwoParameters2) functionToCall).run(resultTime, temp);


  }

  return resultTime;

 }

 // Performs the mutations for Timestamp, adapting to the type of mutation (and parameters that it takes) as needed
 public static Object performTimestamp(java.sql.Timestamp resultTimestamp) throws Throwable {
  int counter_id = 4;

  // Add the mutations to be done in their correct order
  ArrayList < Object > sequence = new ArrayList < Object > ();

  sequence.add(Operators_Timestamp.m1_replaceByNull);
  sequence.add(Operators_Timestamp.m2_replaceByEmptyDummy);
  sequence.add(Operators_Timestamp.m3_replaceByRandom);
  sequence.add(Operators_Timestamp.m4_reverseField);
  sequence.add(Operators_Timestamp.m5_addOneExtraChar);
  sequence.add(Operators_Timestamp.m6_duplicateOneDigit);
  sequence.add(Operators_Timestamp.m7_add100years);
  sequence.add(Operators_Timestamp.m8_subtract100years);
  sequence.add(Operators_Timestamp.m9_removeDigitsFromYear);

  Object functionToCall = null;
  String temp = null;

  synchronized(lock) {
   functionToCall = sequence.get(counter[counter_id]);
  }

  if (functionToCall instanceof Operators_Timestamp.functionOneParameter) {
   synchronized(lock) {
    counter[counter_id]++;
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultTimestamp = ((Operators_Timestamp.functionOneParameter) functionToCall).run(resultTimestamp);


  }

  String[] fieldParameter2 = {
   "year",
   "month",
   "day",
   "hour",
   "minute",
   "second",
   "nano"
  };
  if (functionToCall instanceof Operators_Timestamp.functionTwoParameters) {
   synchronized(lock) {
    temp = fieldParameter2[param1counter[counter_id]];

    param1counter[counter_id]++;
    if (param1counter[counter_id] == fieldParameter2.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultTimestamp = ((Operators_Timestamp.functionTwoParameters) functionToCall).run(resultTimestamp, temp);


  }

  String[] positionParameter6 = {
   "before year",
   "after year",
   "before month",
   "after month",
   "before day",
   "after day",
   "before hour",
   "after hour",
   "before minute",
   "after minute",
   "before second",
   "after second",
   "before nano",
   "after nano"
  };
  if (functionToCall instanceof Operators_Timestamp.functionTwoParameters2) {
   synchronized(lock) {
    temp = positionParameter6[param1counter[counter_id]];

    param1counter[counter_id]++;
    if (param1counter[counter_id] == positionParameter6.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }

   resultTimestamp = ((Operators_Timestamp.functionTwoParameters2) functionToCall).run(resultTimestamp, temp);


  }

  return resultTimestamp;
 }

 // Performs the mutations for Decimal, adapting to the type of mutation (and parameters that it takes) as needed
 public static Object performDecimal(BigDecimal resultBigDecimal) throws Throwable {
  int counter_id = 5;

  // Add the mutations to be done in their correct order
  ArrayList < Object > sequence = new ArrayList < Object > ();

  sequence.add(Operators_Decimal.m1_replaceByNull);
  sequence.add(Operators_Decimal.m2_replaceBySameSizeDecimal);
  sequence.add(Operators_Decimal.m3_add1);
  sequence.add(Operators_Decimal.m4_substract1);
  sequence.add(Operators_Decimal.m5_removeOneNumeric);
  sequence.add(Operators_Decimal.m6_duplicateOneNumeric);
  sequence.add(Operators_Decimal.m7_invertConsecutiveDigits);
  sequence.add(Operators_Decimal.m8_addOneRandomNumeric);
  sequence.add(Operators_Decimal.m9_flipSign);
  sequence.add(Operators_Decimal.m10_zeroOutInteger);
  sequence.add(Operators_Decimal.m11_zeroOutFractional);
  sequence.add(Operators_Decimal.m12_removeIntegerPart);
  sequence.add(Operators_Decimal.m13_removeFractionalPart);
  sequence.add(Operators_Decimal.m14_removeSeparator);
  sequence.add(Operators_Decimal.m15_add10x);
  sequence.add(Operators_Decimal.m16_substract10x);
  sequence.add(Operators_Decimal.m17_shiftDecimalSeparator);
  sequence.add(Operators_Decimal.m18_addZero);
  sequence.add(Operators_Decimal.m19_swapParts);
  sequence.add(Operators_Decimal.m20_reverseFractionalPart);
  sequence.add(Operators_Decimal.m21_reverseIntegerPart);
  sequence.add(Operators_Decimal.m22_skipPointNearZero);

  Object functionToCall = null;
  String temp = null;

  synchronized(lock) {
   functionToCall = sequence.get(counter[counter_id]);
  }

  if (functionToCall instanceof Operators_Decimal.functionOneParameter) {
   synchronized(lock) {
    counter[counter_id]++;
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultBigDecimal = ((Operators_Decimal.functionOneParameter) functionToCall).run(resultBigDecimal);

  }

  String[] direction = {
   "left",
   "right"
  };
  if (functionToCall instanceof Operators_Decimal.functionTwoParameters1) {
   synchronized(lock) {
    temp = direction[param1counter[counter_id]];

    param1counter[counter_id]++;
    if (param1counter[counter_id] == direction.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultBigDecimal = ((Operators_Decimal.functionTwoParameters1) functionToCall).run(resultBigDecimal, temp);

  }

  String[] positionParameter7 = {
   "left",
   "right"
  };
  if (functionToCall instanceof Operators_Decimal.functionTwoParameters2) {
   synchronized(lock) {
    temp = positionParameter7[param1counter[counter_id]];

    param1counter[counter_id]++;
    if (param1counter[counter_id] == positionParameter7.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultBigDecimal = ((Operators_Decimal.functionTwoParameters2) functionToCall).run(resultBigDecimal, temp);

  }

  return resultBigDecimal;
 }

 // Performs the mutations for Double, adapting to the type of mutation (and parameters that it takes) as needed
 public static Object performDouble(double resultDouble) throws Throwable {
  int counter_id = 6;

  // Add the mutations to be done in their correct order
  ArrayList < Object > sequence = new ArrayList < Object > ();

  sequence.add(Operators_Double.m1_replaceBySameSizeDouble);
  sequence.add(Operators_Double.m2_add1);
  sequence.add(Operators_Double.m3_substract1);
  sequence.add(Operators_Double.m4_removeOneNumeric);
  sequence.add(Operators_Double.m5_duplicateOneNumeric);
  sequence.add(Operators_Double.m6_invertConsecutiveDigits);
  sequence.add(Operators_Double.m7_addOneRandomNumeric);
  sequence.add(Operators_Double.m8_flipSign);
  sequence.add(Operators_Double.m9_zeroOutInteger);
  sequence.add(Operators_Double.m10_zeroOutFractional);
  sequence.add(Operators_Double.m11_removeIntegerPart);
  sequence.add(Operators_Double.m12_removeFractionalPart);
  sequence.add(Operators_Double.m13_removeSeparator);
  sequence.add(Operators_Double.m14_add10x);
  sequence.add(Operators_Double.m15_substract10x);
  sequence.add(Operators_Double.m16_shiftDecimalSeparator);
  sequence.add(Operators_Double.m17_addZero);
  sequence.add(Operators_Double.m18_swapParts);
  sequence.add(Operators_Double.m19_reverseFractionalPart);
  sequence.add(Operators_Double.m20_reverseIntegerPart);
  sequence.add(Operators_Double.m21_skipPointNearZero);

  Object functionToCall = null;
  String temp = null;

  synchronized(lock) {
   functionToCall = sequence.get(counter[counter_id]);
  }

  if (functionToCall instanceof Operators_Double.functionOneParameter) {
   synchronized(lock) {
    counter[counter_id]++;
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultDouble = ((Operators_Double.functionOneParameter) functionToCall).run(resultDouble);

  }

  String[] direction = {
   "left",
   "right"
  };
  if (functionToCall instanceof Operators_Double.functionTwoParameters) {
   synchronized(lock) {
    temp = direction[param1counter[counter_id]];

    param1counter[counter_id]++;
    if (param1counter[counter_id] == direction.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultDouble = ((Operators_Double.functionTwoParameters) functionToCall).run(resultDouble, temp);

  }

  String[] positionParameter7 = {
   "left",
   "right"
  };
  if (functionToCall instanceof Operators_Double.functionTwoParameters2) {
   synchronized(lock) {
    temp = positionParameter7[param1counter[counter_id]];

    param1counter[counter_id]++;
    if (param1counter[counter_id] == positionParameter7.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultDouble = ((Operators_Double.functionTwoParameters2) functionToCall).run(resultDouble, temp);

  }

  return resultDouble;
 }

 // Performs the mutations for Boolean, adapting to the type of mutation (and parameters that it takes) as needed
 public static Object performBoolean(boolean resultBoolean) throws Throwable {
  int counter_id = 7;

  // Add the mutations to be done in their correct order
  ArrayList < Object > sequence = new ArrayList < Object > ();
  sequence.add(Operators_Boolean.m1_flipValue);
  sequence.add(Operators_Boolean.m2_setTrue);
  sequence.add(Operators_Boolean.m3_setFalse);

  Object functionToCall = null;
  synchronized(lock) {
   functionToCall = sequence.get(counter[counter_id]);
  }
  if (functionToCall instanceof Operators_Boolean.functionOneParameter) {
   synchronized(lock) {
    counter[counter_id]++;
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultBoolean = ((Operators_Boolean.functionOneParameter) functionToCall).run(resultBoolean);

  }

  return resultBoolean;
 }

 // Performs the mutations for Object, adapting to the type of mutation (and parameters that it takes) as needed
 public static Object performObject(Object result) throws Throwable {
  int counter_id = 8;

  // Add the mutations to be done in their correct order
  ArrayList < Object > sequence = new ArrayList < Object > ();

  sequence.add(Operators_Object.m1_replaceByNull);
  sequence.add(Operators_Object.m2_replaceByEmpty);
  sequence.add(Operators_Object.m3_replaceByDifferentType);

  Object functionToCall = null;
  synchronized(lock) {
   functionToCall = sequence.get(counter[counter_id]);
  }

  if (functionToCall instanceof Operators_Object.functionOneParameter) {
   synchronized(lock) {
    counter[counter_id]++;
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   result = ((Operators_Object.functionOneParameter) functionToCall).run(result);
  }

  return result;
 }

 // Performs the mutations for Ref, adapting to the type of mutation (and parameters that it takes) as needed
 public static Object performRef(Ref resultRef) throws Throwable {
  int counter_id = 9;

  // Add the mutations to be done in their correct order
  ArrayList < Object > sequence = new ArrayList < Object > ();

  sequence.add(Operators_References.m1_replaceByNull);
  sequence.add(Operators_References.m2_replaceDifferentType);

  Object functionToCall = null;
  synchronized(lock) {
   functionToCall = sequence.get(counter[counter_id]);
  }

  if (functionToCall instanceof Operators_References.functionOneParameter) {
   synchronized(lock) {
    counter[counter_id]++;
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultRef = ((Operators_References.functionOneParameter) functionToCall).run(resultRef);

  }

  return resultRef;
 }

 // Performs the mutations for Binary, adapting to the type of mutation (and parameters that it takes) as needed
 public static Object performBinary(byte[] resultBinary) throws Throwable {
  int counter_id = 10;

  // Add the mutations to be done in their correct order
  ArrayList < Object > sequence = new ArrayList < Object > ();

  sequence.add(Operators_Binary.m1_replaceByNull);
  sequence.add(Operators_Binary.m2_replaceByEmpty);
  sequence.add(Operators_Binary.m3_addByte);
  sequence.add(Operators_Binary.m4_removeByte);
  sequence.add(Operators_Binary.m5_duplicateByte);
  sequence.add(Operators_Binary.m6_replaceByte);
  sequence.add(Operators_Binary.m9_flipSign);
  sequence.add(Operators_Binary.m7_dropLeftmostBits);
  sequence.add(Operators_Binary.m8_dropRightmostBits);

  Object functionToCall = null;
  synchronized(lock) {
   functionToCall = sequence.get(counter[counter_id]);
  }

  if (functionToCall instanceof Operators_Binary.functionOneParameter) {
   synchronized(lock) {
    counter[counter_id]++;
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultBinary = ((Operators_Binary.functionOneParameter) functionToCall).run(resultBinary);

  }


  String[] position = {
   "prefix",
   "suffix",
   "middle"
  };
  if (functionToCall instanceof Operators_Binary.functionTwoParameters) {
   String temp;
   synchronized(lock) {
    temp = position[param1counter[counter_id]];
    param1counter[counter_id]++;
    if (param1counter[counter_id] == position.length) {
     param1counter[counter_id] = 0;
     counter[counter_id]++;
    }
    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultBinary = ((Operators_Binary.functionTwoParameters) functionToCall).run(resultBinary, temp);

  }

  if (functionToCall instanceof Operators_Binary.functionTwoParameters2) {
   Random random = new Random();
   int param = random.nextInt(8) + 1;
   synchronized(lock) {

    counter[counter_id]++;

    if (counter[counter_id] == sequence.size()) {
     counter[counter_id] = 0;
    }
   }
   resultBinary = ((Operators_Binary.functionTwoParameters2) functionToCall).run(resultBinary, param);

  }

  return resultBinary;
 }
}