// Poor Data Injector
// 2015-2016 Seyma Nur Soydemir seyma@student.dei.uc.pt

package intercept;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.math.BigDecimal;
import java.sql.Ref;
import java.sql.Timestamp;
import java.util.ArrayList;

import mutation.Operators_Binary;
import mutation.Operators_Boolean;
import mutation.Operators_Date;
import mutation.Operators_Decimal;
import mutation.Operators_Double;
import mutation.Operators_Integer;
import mutation.Operators_Object;
import mutation.Operators_References;
import mutation.Operators_String;
import mutation.Operators_Time;
import mutation.Operators_Timestamp;

import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;

@Aspect
public class MyAspect {

 // Path to where the log of mutations (i.e, mutationlog) will be stored
 private static String static_mutationlog_path = "C:\\mutationlog.txt";

 // Path to the file that enables/disables the mutations
 private static String static_enablefile_path = "D:\\Seyma\\thesis\\logs\\EnableFile.txt";

 // Path to the file that has the stack-trace lines that should be ignored when doing mutations
 private static String static_errorlines_path = "D:\\Seyma\\thesis\\logs\\ErrorLines.txt";


 // Signals whether we are injecting or not
 public static boolean injection = true;

 // Signals that the user has enabled the injections (in EnableFile)
 private static boolean enabled = false;

 private static PrintWriter writer = null;

 // Array with all the strack-trace lines that should be skipped during the mutations
 public static ArrayList < String > errorLines = null;

 // Helper function to write a string to file
 public void writeToFile(String content) {
  try {
   if (writer == null) {
    writer = new PrintWriter(new FileOutputStream(static_mutationlog_path, true));
   }
  } catch (FileNotFoundException e) {
   e.printStackTrace();
  }
  writer.println(content);
  writer.flush();
 }

 // Helper function to write an exception to file
 public void writeExceptionToFile(Exception ea) {
  try {
   if (writer == null) {
    writer = new PrintWriter(new FileOutputStream(static_mutationlog_path, true));
   }
  } catch (FileNotFoundException e) {
   e.printStackTrace();
  }
  ea.printStackTrace(writer);
  writer.flush();
 }


 // Reads the enable file (i.e., the file that signals whether the tool can inject or not)
 public synchronized void readEnableFile() {
  try {
   BufferedReader reader = new BufferedReader(new FileReader(static_enablefile_path));
   String line;
   while ((line = reader.readLine()) != null) {
    // If the file has 1 inside, we start mutations
    if (line.equalsIgnoreCase("1")) {
     enabled = true;
    }
    // If it has 0, we stop mutations
    else if (line.equalsIgnoreCase("0")) {
     enabled = false;
    }
   }
   reader.close();
  } catch (Exception e) {
   System.err.format("Exception occurred trying to read file");
   e.printStackTrace();
  }

 }

 // Read the file that has all the stack-trace lines that should be ignored when doing mutations
 public synchronized void readErrorLines() {
  if (errorLines == null) {
   errorLines = new ArrayList < String > ();
   try {
    BufferedReader reader = new BufferedReader(new FileReader(static_errorlines_path));
    String tmp_group = "";
    String line;
    while ((line = reader.readLine()) != null) {
     // Each set of error lines is separated by a line with "*END OF ENTRY*"
     if (line.equalsIgnoreCase("*END OF ENTRY*")) {
      errorLines.add(tmp_group);
      tmp_group = "";
     } else {
      // Remove unwanted and possibly troubling characters
      line = line.replaceAll("\\r|\\n", "");
      line = line.replace("\t", "");
      line = line.replace(" ", "");

      tmp_group = tmp_group + line;
     }
    }
    if (!tmp_group.equalsIgnoreCase("")) {
     errorLines.add(tmp_group);
    }
    reader.close();
   } catch (Exception e) {
    System.err.format("Exception occurred trying to read file");
    e.printStackTrace();
   }

  }
 }


 // Helper function to obtain the stack trace of an exception
 public String readException(Exception ex) {
  StringWriter errors = new StringWriter();
  ex.printStackTrace(new PrintWriter(errors));
  return errors.toString();
 }


 // Helper function to find the stack-trace of an exception will all the stack-traces in ErrorLines
 public boolean compareException(String exc) {

  // Remove unwanted and possibly troubling characters
  exc = exc.replaceAll("\\r|\\n", "");
  exc = exc.replace("\t", "");
  exc = exc.replace(" ", "");

  for (String temp: errorLines) {

   if (exc.indexOf(temp) >= 0) {
    return true;
   }
  }
  return false;
 }

 // A global lock object to ensure that two mutations cannot happen at the same time (to ensure the correct function of the tool)
 public static Object lock = new Object();

 // Run this function whenever getString() of JDBC driver is called (e.g., to obtain a String value from database)
 // This function will handle the entire mutation process
 @Around("execution(* java.sql.ResultSet.getString(..))")
 public synchronized Object logAroundString(ProceedingJoinPoint joinPoint) throws Throwable {

  Object result = null;
  String stackTrace = null;
  try {
   result = joinPoint.proceed();
  } catch (Exception ab) {
   writeToFile("Exception on Application Execution!");
   writeExceptionToFile(ab);
  }

  if (injection == false) {
   return result;
  }
  readEnableFile();
  if (enabled == false) {
   return result;
  }

  readErrorLines();
  Exception exc = new Exception("Stack-Trace");
  stackTrace = readException(exc);
  if (compareException(stackTrace)) {
   System.out.println("Skipped JDBC call!");

   return result;
  }

  String resultString = (String) result;
  writeToFile("Time: " + new Timestamp(new java.util.Date().getTime()));
  writeToFile("Stack-Trace of Mutation caller");
  writeExceptionToFile(exc);
  writeToFile("Input (before mutation): " + resultString);

  // Temporarily store old counters of MutationPerformer, because we might need to restore it
  int tmp_counter = MutationPerformer.counter[0];
  int tmp_param1Counter = MutationPerformer.param1counter[0];
  int tmp_param2Counter = MutationPerformer.param2counter[0];

  try {
   resultString = (String) MutationPerformer.performString(resultString);
  } catch (Exception e) {
   writeToFile("Exception on String Mutation!");
   writeExceptionToFile(e);
   // Continue

  }

  // The code below is an add on added during our experiments with Openbravo. It is used to avoid a programming
  // bug in the ORM. For most cases this code can be removed.
  boolean input_y_n = false;
  if ((input_y_n == true)) {
   // Restore previous state before this mutation, so it can be done again
   MutationPerformer.counter[0] = tmp_counter;
   MutationPerformer.param1counter[0] = tmp_param1Counter;
   MutationPerformer.param2counter[0] = tmp_param2Counter;
   resultString = (String) result;
   writeToFile("Skipped to avoid ORM bug");
  } else {
   writeToFile("Used Mutation: String_" + Operators_String.nameOfLastExecutedFunction);
   writeToFile("Output (after mutation): " + resultString + "\n\n");
  }
  Operators_String.nameOfLastExecutedFunction = null;

  return resultString;

 }

 // Run this function whenever getLong() of JDBC driver is called (e.g., to obtain a Long value from database)
 // This function will handle the entire mutation process
 @Around("execution(* java.sql.ResultSet.getLong(..))")
 public synchronized Object logAroundLong(ProceedingJoinPoint joinpoint) throws Throwable {
  Object result = null;
  String stackTrace = null;

  try {
   result = joinpoint.proceed();
  } catch (Exception ab) {
   writeToFile("Exception on Application Execution!");
   writeExceptionToFile(ab);
  }

  if (injection == false) {
   return result;
  }

  readEnableFile();
  if (enabled == false) {
   return result;
  }

  readErrorLines();
  Exception exc = new Exception("Stack-Trace");
  stackTrace = readException(exc);
  if (compareException(stackTrace)) {
   System.out.println("Skipped JDBC call!");

   return result;
  }


  Long resultLong = (Long) result;
  writeToFile("Time: " + new Timestamp(new java.util.Date().getTime()));

  writeToFile("Stack-Trace of Mutation caller");
  writeExceptionToFile(exc);
  writeToFile("Input (before mutation): " + resultLong);

  try {
   resultLong = (Long) MutationPerformer.performLong(resultLong);
  } catch (Exception e) {
   writeToFile("Exception on Long Mutation!");
   writeExceptionToFile(e);
   // Continue
  }

  writeToFile("Used Mutation:Long Integer_" + Operators_Integer.nameOfLastExecutedFunction);
  writeToFile("Output (after mutation): " + resultLong);

  Operators_Integer.nameOfLastExecutedFunction = null;


  return resultLong;
 }

 // Run this function whenever getInt() of JDBC driver is called (e.g., to obtain an Integer value from database)
 // This function will handle the entire mutation process
 @Around("execution(* java.sql.ResultSet.getInt(..))")
 public synchronized Object logAroundInt(ProceedingJoinPoint joinpoint) throws Throwable {
  Object result = null;
  String stackTrace = null;

  try {
   result = joinpoint.proceed();
  } catch (Exception ab) {
   writeToFile("Exception on Application Execution!");
   writeExceptionToFile(ab);
  }

  if (injection == false) {
   return result;
  }

  readEnableFile();
  if (enabled == false) {
   return result;
  }

  readErrorLines();
  Exception exc = new Exception("Stack-Trace");
  stackTrace = readException(exc);
  if (compareException(stackTrace)) {
   System.out.println("Skipped JDBC call!");

   return result;
  }

  Long resultLong = new Long((Integer) result);

  writeToFile("Time: " + new Timestamp(new java.util.Date().getTime()));
  writeToFile("Stack-Trace of Mutation caller");
  writeExceptionToFile(exc);
  writeToFile("Input (before mutation): " + resultLong);

  try {
   resultLong = (Long) MutationPerformer.performLong(resultLong);
  } catch (Exception e) {
   writeToFile("Exception on Int Mutation!");
   writeExceptionToFile(e);
   // Continue
  }

  writeToFile("Used Mutation:Int Integer_" + Operators_Integer.nameOfLastExecutedFunction);
  writeToFile("Output (after mutation): " + resultLong);

  Operators_Integer.nameOfLastExecutedFunction = null;

  return resultLong;
 }

 // Run this function whenever getShort() of JDBC driver is called (e.g., to obtain a short integer value from database)
 // This function will handle the entire mutation process
 @Around("execution(* java.sql.ResultSet.getShort(..))")
 public synchronized Object logAroundShort(ProceedingJoinPoint joinpoint) throws Throwable {
  Object result = null;
  String stackTrace = null;

  try {
   result = joinpoint.proceed();
  } catch (Exception ab) {
   writeToFile("Exception on Application Execution!");
   writeExceptionToFile(ab);
  }

  if (injection == false) {
   return result;
  }
  readEnableFile();
  if (enabled == false) {
   return result;
  }

  readErrorLines();
  Exception exc = new Exception("Stack-Trace");
  stackTrace = readException(exc);
  if (compareException(stackTrace)) {
   System.out.println("Skipped JDBC call!");

   return result;
  }

  Long resultLong = new Long((Short) result);
  writeToFile("Time: " + new Timestamp(new java.util.Date().getTime()));
  writeToFile("Stack-Trace of Mutation caller");
  writeExceptionToFile(exc);
  writeToFile("Input (before mutation): " + resultLong);

  try {
   resultLong = (Long) MutationPerformer.performLong(resultLong);
  } catch (Exception e) {
   writeToFile("Exception on Long Mutation!");
   writeExceptionToFile(e);
   // Continue
  }

  writeToFile("Used Mutation:Short Integer_" + Operators_Integer.nameOfLastExecutedFunction);
  writeToFile("Output (after mutation): " + resultLong);

  Operators_Integer.nameOfLastExecutedFunction = null;


  return resultLong;
 }

 // Run this function whenever getByte() of JDBC driver is called (e.g., to obtain a byte integer value from database)
 // This function will handle the entire mutation process
 @Around("execution(* java.sql.ResultSet.getByte(..))")
 public synchronized Object logAroundByte(ProceedingJoinPoint joinpoint) throws Throwable {
  Object result = null;
  String stackTrace = null;

  try {
   result = joinpoint.proceed();
  } catch (Exception ab) {
   writeToFile("Exception on Application Execution!");
   writeExceptionToFile(ab);
  }

  if (injection == false) {
   return result;
  }
  readEnableFile();
  if (enabled == false) {
   return result;
  }

  readErrorLines();
  Exception exc = new Exception("Stack-Trace");
  stackTrace = readException(exc);
  if (compareException(stackTrace)) {
   System.out.println("Skipped JDBC call!");

   return result;
  }

  Long resultLong = new Long((Byte) result);
  writeToFile("Time: " + new Timestamp(new java.util.Date().getTime()));
  writeToFile("Stack-Trace of Mutation caller");
  writeExceptionToFile(exc);
  writeToFile("Input (before mutation): " + resultLong);

  try {
   resultLong = (Long) MutationPerformer.performLong(resultLong);
  } catch (Exception e) {
   writeToFile("Exception on Long Mutation!");
   writeExceptionToFile(e);
   // Continue
  }

  writeToFile("Used Mutation:Byte Integer_" + Operators_Integer.nameOfLastExecutedFunction);
  writeToFile("Output (after mutation): " + resultLong);

  Operators_Integer.nameOfLastExecutedFunction = null;


  return resultLong;
 }


 // Run this function whenever getDate() of JDBC driver is called (e.g., to obtain a Date value from database)
 // This function will handle the entire mutation process
 @Around("execution(* java.sql.ResultSet.getDate(..))")
 public synchronized Object logAroundDate(ProceedingJoinPoint joinpoint) throws Throwable {
  readErrorLines();
  Object result = null;
  String stackTrace = null;

  try {
   result = joinpoint.proceed();
  } catch (Exception ab) {
   writeToFile("Exception on Application Execution!");
   writeExceptionToFile(ab);
  }

  if (injection == false) {
   return result;
  }
  readEnableFile();
  if (enabled == false) {
   return result;
  }

  readErrorLines();
  Exception exc = new Exception("Stack-Trace");
  stackTrace = readException(exc);
  if (compareException(stackTrace)) {
   System.out.println("Skipped JDBC call!");

   return result;
  }

  java.sql.Date resultDate = (java.sql.Date) result;

  writeToFile("Time: " + new Timestamp(new java.util.Date().getTime()));
  writeToFile("Stack-Trace of Mutation caller");
  writeExceptionToFile(exc);


  writeToFile("Input (before mutation): " + resultDate);
  try {
   resultDate = (java.sql.Date) MutationPerformer.performDate(resultDate);
  } catch (Exception e) {
   writeToFile("Exception on Date Mutation!");
   writeExceptionToFile(e);
   // Continue
  }

  writeToFile("Used Mutation: Date_" + Operators_Date.nameOfLastExecutedFunction);
  writeToFile("Output (after mutation): " + resultDate);

  Operators_Date.nameOfLastExecutedFunction = null;


  return resultDate;
 }


 // Run this function whenever getTime() of JDBC driver is called (e.g., to obtain a Time value from database)
 // This function will handle the entire mutation process
 @Around("execution(* java.sql.ResultSet.getTime(..))")
 public synchronized Object logAroundTime(ProceedingJoinPoint joinpoint) throws Throwable {
  readErrorLines();
  Object result = null;
  String stackTrace = null;

  try {
   result = joinpoint.proceed();
  } catch (Exception ab) {
   writeToFile("Exception on Application Execution!");
   writeExceptionToFile(ab);
  }

  if (injection == false) {
   return result;
  }
  readEnableFile();
  if (enabled == false) {
   return result;
  }

  readErrorLines();
  Exception exc = new Exception("Stack-Trace");
  stackTrace = readException(exc);
  if (compareException(stackTrace)) {
   System.out.println("Skipped JDBC call!");

   return result;
  }

  java.sql.Time resultTime = (java.sql.Time) result;


  writeToFile("Time: " + new Timestamp(new java.util.Date().getTime()));
  writeToFile("Stack-Trace of Mutation caller");
  writeExceptionToFile(exc);

  writeToFile("Input (before mutation): " + resultTime);
  try {
   resultTime = (java.sql.Time) MutationPerformer.performTime(resultTime);
  } catch (Exception e) {
   writeToFile("Exception on Time Mutation!");
   writeExceptionToFile(e);
   // Continue
  }

  writeToFile("Used Mutation: Time_" + Operators_Time.nameOfLastExecutedFunction);
  writeToFile("Output (after mutation): " + resultTime);

  Operators_Time.nameOfLastExecutedFunction = null;


  return resultTime;

 }


 // Run this function whenever getTimestamp() of JDBC driver is called (e.g., to obtain a Timestamp value from database)
 // This function will handle the entire mutation process
 @Around("execution(* java.sql.ResultSet.getTimestamp(..))")
 public synchronized Object logAroundTimestamp(ProceedingJoinPoint joinpoint) throws Throwable {
  String stackTrace = null;
  Object result = null;
  try {
   result = joinpoint.proceed();
  } catch (Exception ab) {
   writeToFile("Exception on Application Execution!");
   writeExceptionToFile(ab);
  }

  if (injection == false) {
   return result;
  }
  readEnableFile();
  if (enabled == false) {
   return result;
  }

  readErrorLines();
  Exception exc = new Exception("Stack-Trace");
  stackTrace = readException(exc);
  if (compareException(stackTrace)) {
   System.out.println("Skipped JDBC call!");

   return result;
  }

  java.sql.Timestamp resultTimestamp = (java.sql.Timestamp) result;


  writeToFile("Time: " + new Timestamp(new java.util.Date().getTime()));
  writeToFile("Stack-Trace of Mutation caller");
  writeExceptionToFile(exc);

  writeToFile("Input (before mutation): " + resultTimestamp);

  try {
   resultTimestamp = (java.sql.Timestamp) MutationPerformer.performTimestamp(resultTimestamp);
  } catch (Exception e) {
   writeToFile("Exception on Timestamp Mutation!");
   writeExceptionToFile(e);
   // Continue

  }
  writeToFile("Used Mutation: Timestamp_" + Operators_Timestamp.nameOfLastExecutedFunction);
  writeToFile("Output (after mutation): " + resultTimestamp);

  Operators_Timestamp.nameOfLastExecutedFunction = null;

  return resultTimestamp;
 }


 // Run this function whenever getBigDecimal() of JDBC driver is called (e.g., to obtain a decimal value from database)
 // This function will handle the entire mutation process
 @Around("execution(* java.sql.ResultSet.getBigDecimal(..))")
 public synchronized Object logAroundDecimal(ProceedingJoinPoint joinpoint) throws Throwable {
  Object result = null;
  String stackTrace = null;
  try {
   result = joinpoint.proceed();
  } catch (Exception ab) {
   writeToFile("Exception on Application Execution!");
   writeExceptionToFile(ab);
  }

  if (injection == false) {
   return result;
  }
  readEnableFile();
  if (enabled == false) {
   return result;
  }

  readErrorLines();
  Exception exc = new Exception("Stack-Trace");
  stackTrace = readException(exc);
  if (compareException(stackTrace)) {
   System.out.println("Skipped JDBC call!");

   return result;
  }

  BigDecimal resultBigDecimal = (BigDecimal) result;


  writeToFile("Time: " + new Timestamp(new java.util.Date().getTime()));
  writeToFile("Stack-Trace of Mutation caller");
  writeExceptionToFile(exc);

  writeToFile("Input (before mutation): " + resultBigDecimal);

  try {
   resultBigDecimal = (BigDecimal) MutationPerformer.performDecimal(resultBigDecimal);
  } catch (Exception e) {
   writeToFile("Exception on Decimal Mutation!");
   writeExceptionToFile(e);
   // Continue

  }

  writeToFile("Used Mutation: Decimal_" + Operators_Decimal.nameOfLastExecutedFunction);
  writeToFile("Output (after mutation): " + resultBigDecimal);

  Operators_Decimal.nameOfLastExecutedFunction = null;

  return resultBigDecimal;
 }


 // Run this function whenever getDouble() of JDBC driver is called (e.g., to obtain a Double value from database)
 // This function will handle the entire mutation process
 @Around("execution(* java.sql.ResultSet.getDouble(..))")
 public synchronized Object logAroundDouble(ProceedingJoinPoint joinpoint) throws Throwable {

  Object result = null;
  String stackTrace = null;

  try {
   result = joinpoint.proceed();
  } catch (Exception ab) {
   writeToFile("Exception on Application Execution!");
   writeExceptionToFile(ab);
  }

  if (injection == false) {
   return result;
  }
  readEnableFile();
  if (enabled == false) {
   return result;
  }

  readErrorLines();
  Exception exc = new Exception("Stack-Trace");
  stackTrace = readException(exc);
  if (compareException(stackTrace)) {
   System.out.println("Skipped JDBC call!");

   return result;
  }

  double resultDouble = (Double) result;

  writeToFile("Time: " + new Timestamp(new java.util.Date().getTime()));
  //writeToFile("File:" + joinPoint.getSourceLocation().getFileName());
  //writeToFile("Line:" + joinPoint.getSourceLocation().getLine());
  writeToFile("Stack-Trace of Mutation caller");
  writeExceptionToFile(exc);

  writeToFile("Input (before mutation): " + resultDouble);

  try {
   resultDouble = (Double) MutationPerformer.performDouble(resultDouble);
  } catch (Exception e) {
   writeToFile("Exception on Double Mutation!");
   writeExceptionToFile(e);
   // Continue

  }

  writeToFile("Used Mutation: Double_" + Operators_Double.nameOfLastExecutedFunction);
  writeToFile("Output (after mutation): " + resultDouble);

  Operators_Double.nameOfLastExecutedFunction = null;

  return resultDouble;
 }

 // Run this function whenever getBoolean() of JDBC driver is called (e.g., to obtain a boolean value from database)
 // This function will handle the entire mutation process
 @Around("execution(* java.sql.ResultSet.getBoolean(..))")
 public synchronized Object logAroundBoolean(ProceedingJoinPoint joinpoint) throws Throwable {

  String stackTrace = null;
  Object result = null;
  try {
   result = joinpoint.proceed();
  } catch (Exception ab) {
   writeToFile("Exception on Application Execution!");
   writeExceptionToFile(ab);
  }

  if (injection == false) {
   return result;
  }
  readEnableFile();
  if (enabled == false) {
   return result;
  }

  readErrorLines();
  Exception exc = new Exception("Stack-Trace");
  stackTrace = readException(exc);
  if (compareException(stackTrace)) {
   System.out.println("Skipped JDBC call!");

   return result;
  }

  boolean resultBoolean = (Boolean) result;

  writeToFile("Time: " + new Timestamp(new java.util.Date().getTime()));
  writeToFile("Stack-Trace of Mutation caller");
  writeExceptionToFile(exc);

  writeToFile("Input (before mutation): " + resultBoolean);

  try {
   resultBoolean = (Boolean) MutationPerformer.performBoolean(resultBoolean);
  } catch (Exception e) {
   writeToFile("Exception on Boolean Mutation!");
   writeExceptionToFile(e);
   // Continue

  }

  writeToFile("Used Mutation: Boolean_" + Operators_Boolean.nameOfLastExecutedFunction);
  writeToFile("Output (after mutation): " + resultBoolean);

  return resultBoolean;
 }



 // Run this function whenever getObject() of JDBC driver is called (e.g., to obtain an Object value from database)
 // This function will handle the entire mutation process
 @Around("execution(* java.sql.ResultSet.getObject(..))")
 public synchronized Object logAroundObject(ProceedingJoinPoint joinpoint) throws Throwable {
  String stackTrace = null;
  Object result = null;
  try {
   result = joinpoint.proceed();
  } catch (Exception ab) {
   writeToFile("Exception on Application Execution!");
   writeExceptionToFile(ab);
  }

  if (injection == false) {
   return result;
  }
  readEnableFile();
  if (enabled == false) {
   return result;
  }

  readErrorLines();
  Exception exc = new Exception("Stack-Trace");
  stackTrace = readException(exc);
  if (compareException(stackTrace)) {
   System.out.println("Skipped JDBC call!");

   return result;
  }

  writeToFile("Time: " + new Timestamp(new java.util.Date().getTime()));
  writeToFile("Stack-Trace of Mutation caller");
  writeExceptionToFile(exc);


  writeToFile("Input (before mutation): " + result);
  try {
   result = MutationPerformer.performObject(result);
  } catch (Exception e) {
   writeToFile("Exception on String Mutation!");
   writeExceptionToFile(e);
   // Continue
  }

  writeToFile("Used Mutation: Object_" + Operators_Object.nameOfLastExecutedFunction);
  writeToFile("Output (after mutation): " + result);

  Operators_Object.nameOfLastExecutedFunction = null;

  return result;
 }



 // Run this function whenever getRef() of JDBC driver is called (e.g., to obtain a Ref value from database)
 // This function will handle the entire mutation process
 @Around("execution(* java.sql.ResultSet.getRef(..))")
 public synchronized Object logAroundRef(ProceedingJoinPoint joinpoint) throws Throwable {

  String stackTrace = null;
  Object result = null;
  try {
   result = joinpoint.proceed();
  } catch (Exception ab) {
   writeToFile("Exception on Application Execution!");
   writeExceptionToFile(ab);
  }

  if (injection == false) {
   return result;
  }
  readEnableFile();
  if (enabled == false) {
   return result;
  }

  readErrorLines();
  Exception exc = new Exception("Stack-Trace");
  stackTrace = readException(exc);
  if (compareException(stackTrace)) {
   System.out.println("Skipped JDBC call!");

   return result;
  }

  Ref resultRef = (Ref) result;

  writeToFile("Time: " + new Timestamp(new java.util.Date().getTime()));
  writeToFile("Stack-Trace of Mutation caller");
  writeExceptionToFile(exc);

  writeToFile("Input (before mutation): " + resultRef);

  try {
   resultRef = (Ref) MutationPerformer.performRef(resultRef);
  } catch (Exception e) {
   writeToFile("Exception on Reference Mutation!");
   writeExceptionToFile(e);
   // Continue
  }

  writeToFile("Used Mutation: References_" + Operators_References.nameOfLastExecutedFunction);
  writeToFile("Output (after mutation): " + result);

  Operators_References.nameOfLastExecutedFunction = null;

  return resultRef;
 }


 // Run this function whenever getTimestamp() of JDBC driver is called (e.g., to obtain a Binary value from database)
 // This function will handle the entire mutation process
 @Around("execution(* java.sql.ResultSet.getBytes(..))")
 public synchronized Object logAroundBinary(ProceedingJoinPoint joinpoint) throws Throwable {
  String stackTrace = null;
  Object result = null;
  try {
   result = joinpoint.proceed();
  } catch (Exception ab) {
   writeToFile("Exception on Application Execution!");
   writeExceptionToFile(ab);
  }

  if (injection == false) {
   return result;
  }
  readEnableFile();
  if (enabled == false) {
   return result;
  }

  readErrorLines();
  Exception exc = new Exception("Stack-Trace");
  stackTrace = readException(exc);
  if (compareException(stackTrace)) {
   System.out.println("Skipped JDBC call!");
   return result;
  }

  byte[] resultBin = (byte[]) result;

  writeToFile("Time: " + new Timestamp(new java.util.Date().getTime()));
  writeToFile("Stack-Trace of Mutation caller");
  writeExceptionToFile(exc);

  writeToFile("Input (before mutation): " + resultBin);

  try {
   resultBin = (byte[]) MutationPerformer.performBinary(resultBin);
  } catch (Exception e) {
   writeToFile("Exception on Binary Mutation!");
   writeExceptionToFile(e);
   // Continue
  }

  writeToFile("Used Mutation: Binary_" + Operators_Binary.nameOfLastExecutedFunction);
  writeToFile("Output (after mutation): " + result);

  Operators_Binary.nameOfLastExecutedFunction = null;

  return resultBin;
 }

}