// Poor Data Injector
// 2015-2016 Seyma Nur Soydemir seyma@student.dei.uc.pt

package mutation;

import java.sql.Date;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Random;

public class Operators_Date {

 // Variable that holds the name of the last executed mutation
 public static String nameOfLastExecutedFunction = null;

 // Interfaces for the various mutations
 public interface functionOneParameter {
  java.sql.Date run(java.sql.Date input);
 }

 public interface functionTwoParameters {
  java.sql.Date run(java.sql.Date input, String position);
 }

 public interface functionTwoParameters2 {
  java.sql.Date run(java.sql.Date input, String position);
 }

 public interface functionTwoParameters3 {
  java.sql.Date run(java.sql.Date input, java.sql.Date defaultDate);
 }

 public interface functionThreeParameters {
  java.sql.Date run(java.sql.Date input, String source, String target);
 }


 // Static pre-defined values
 static int maximum_allowed_year_field = 9999;
 static int minimum_allowed_year_field = -1900;

 static int maximum_allowed_month_field = 11;
 static int minimum_allowed_month_field = 0;

 static int maximum_allowed_day_field = 31;
 static int minimum_allowed_day_field = 1;

 // Obtain maximum days in a month of a certain year
 public static int getMaximumDaysOfMonth(int year, int month) {
  // Date's year 0 == 1900, but GregorianCalendar's year 0 == 1700
  Calendar mycal = new GregorianCalendar(year + (1900 - 1700), month, 1);

  return mycal.getActualMaximum(Calendar.DAY_OF_MONTH); // 28
 }

 // Method 1: Replace by null value
 public static final functionOneParameter m1_replaceByNull = new functionOneParameter() {

  @Override
  public Date run(Date input) {

   nameOfLastExecutedFunction = "m1_replaceByNull";

   return null;
  }
 };

 // Method 2: Replace by empty dummy date
 public static final functionOneParameter m2_replaceByEmptyDummyDate = new functionOneParameter() {

  @Override
  public Date run(Date input) {
   nameOfLastExecutedFunction = "m2_replaceByEmptyDummyDate";

   java.sql.Date dummy = new java.sql.Date(0, 0, 1);

   return dummy;
  }
 };

 // Method 3: Replace by random date 
 // Parameters:
 // 	Position: Year - Month - Day
 public static final functionTwoParameters m3_replaceByRandomDate = new functionTwoParameters() {

  @Override
  public Date run(Date input, String position) {
   nameOfLastExecutedFunction = "m3_replaceByRandomDate";

   Random random = new Random();
   java.sql.Date output = (Date) input.clone();

   if (position.equalsIgnoreCase("Year")) {
    int randomYear = random.nextInt(maximum_allowed_year_field - minimum_allowed_year_field + 1) + minimum_allowed_year_field;
    output.setYear(randomYear);
   }

   if (position.equalsIgnoreCase("Month")) {
    int randomMonth = random.nextInt(maximum_allowed_month_field - minimum_allowed_month_field + 1) + minimum_allowed_month_field;
    output.setMonth(randomMonth);
   }

   if (position.equalsIgnoreCase("Day")) {
    int randomDay = random.nextInt(getMaximumDaysOfMonth(
     input.getYear(), input.getMonth())) + 1;
    output.setDate(randomDay);
   }

   return output;
  }
 };

 // Method 4: Add one extra digit to date
 // Parameters:
 // 	Position: Before year value - After year value - Before month value -
 // 			  After month value - Before day value - After day value
 public static final functionTwoParameters2 m4_addOneExtra = new functionTwoParameters2() {

  @Override
  public Date run(Date input, String position) {
   nameOfLastExecutedFunction = "m4_addOneExtra";

   java.sql.Date output = (Date) input.clone();

   Random random = new Random();

   if (position.equalsIgnoreCase("Before year value")) {
    int currentYear = input.getYear() + 1900;
    String currentYearStr = String.valueOf(currentYear);
    int newYear = currentYear;
    while ((currentYear == newYear)) {
     int randomDigit = random.nextInt(10);
     String randomStr = String.valueOf(randomDigit);
     String temp = randomStr + currentYearStr;
     newYear = Integer.parseInt(temp);
    }
    output.setYear(newYear - 1900);
    return output;
   }

   if (position.equalsIgnoreCase("After year value")) {
    int currentYear = input.getYear() + 1900;
    String currentYearStr = String.valueOf(currentYear);
    int newYear = currentYear;
    while ((currentYear == newYear)) {
     int randomDigit = random.nextInt(10);
     String randomStr = String.valueOf(randomDigit);
     String temp = currentYearStr + randomStr;
     newYear = Integer.parseInt(temp);
    }
    output.setYear(newYear - 1900);
    return output;
   }

   if (position.equalsIgnoreCase("Before month value")) {
    int currentMonth = input.getMonth() + 1;
    int newMonth = 123;
    String currentMonthStr = String.valueOf(currentMonth);
    while ((newMonth > 12) || (currentMonth == newMonth)) {
     int randomDigit = random.nextInt(10);
     String randomStr = String.valueOf(randomDigit);
     String temp = randomStr + currentMonthStr;
     newMonth = Integer.parseInt(temp);
    }
    output.setMonth(newMonth - 1);
    return output;
   }

   if (position.equalsIgnoreCase("After month value")) {

    int currentMonth = input.getMonth() + 1;
    int newMonth = 123;
    String currentMonthStr = String.valueOf(currentMonth);
    while ((newMonth > 12) || (currentMonth == newMonth)) {
     int randomDigit = random.nextInt(10);
     String randomStr = String.valueOf(randomDigit);
     String temp = currentMonthStr + randomStr;
     newMonth = Integer.parseInt(temp);
    }
    output.setMonth(newMonth - 1);
    return output;
   }

   if (position.equalsIgnoreCase("Before day value")) {
    int currentDay = input.getDate();
    String currentDayStr = String.valueOf(currentDay);
    int newDay = 213;

    while (newDay > getMaximumDaysOfMonth(input.getYear(),
      input.getMonth()) || (currentDay == newDay)) {
     int randomDigit = random.nextInt(10);
     String randomStr = String.valueOf(randomDigit);
     String temp = randomStr + currentDayStr;
     newDay = Integer.parseInt(temp);

    }
    output.setDate(newDay);
    return output;
   }

   if (position.equalsIgnoreCase("After day value")) {
    int currentDay = input.getDate();
    String currentDayStr = String.valueOf(currentDay);
    int newDay = currentDay;

    while (newDay > getMaximumDaysOfMonth(input.getYear(),
      input.getMonth()) || (currentDay == newDay)) {
     int randomDigit = random.nextInt(10);
     String randomStr = String.valueOf(randomDigit);
     String temp = currentDayStr + randomStr;
     newDay = Integer.parseInt(temp);

    }
    output.setDate(newDay);
    return output;
   }
   return input;
  }
 };

 // Method 5: Remove one digit from date
 // Parameters:
 // 	Position: year - month - day
 public static final functionTwoParameters m5_removeOneDigit = new functionTwoParameters() {

  @Override
  public Date run(Date input, String position) {
   nameOfLastExecutedFunction = "m5_removeOneDigit";

   java.sql.Date output = (Date) input.clone();
   Random random = new Random();
   int randomPosition = 0;
   if (position.equalsIgnoreCase("year")) {
    int year = input.getYear() + 1900;
    String yearStr = String.valueOf(year);
    randomPosition = random.nextInt(yearStr.length());
    yearStr = yearStr.substring(0, randomPosition) + yearStr.substring(randomPosition + 1);
    output.setYear(Integer.parseInt(yearStr) - 1900);
    return output;
   }

   if (position.equalsIgnoreCase("month")) {
    int month = input.getMonth() + 1;
    String monthStr = String.valueOf(month);
    int newMonth = month;
    while (newMonth == 0 || newMonth == month) {
     randomPosition = random.nextInt(monthStr.length());
     monthStr = monthStr.substring(0, randomPosition) + monthStr.substring(randomPosition + 1);
     newMonth = Integer.parseInt(monthStr);
    }
    output.setMonth(newMonth - 1);
    return output;
   }

   if (position.equalsIgnoreCase("day")) {
    int day = input.getDate();
    String dayStr = String.valueOf(day);
    int newDay = day;
    while (newDay == 0 || newDay == day) {
     randomPosition = random.nextInt(dayStr.length());
     dayStr = dayStr.substring(0, randomPosition) + dayStr.substring(randomPosition + 1);
     newDay = Integer.parseInt(dayStr);
    }
    output.setDate(newDay);
    return output;
   }

   return input;
  }
 };


 // Method 6: Replace one digit in date with a random character
 // Parameters:
 // 	Position: year - month - day
 public static final functionTwoParameters m6_replaceDigitWithRandom = new functionTwoParameters() {

  @Override
  public Date run(Date input, String position) {
   nameOfLastExecutedFunction = "m6_replaceDigitWithRandom";

   java.sql.Date output = (Date) input.clone();
   Random random = new Random();
   int randomPosition = 0;
   int randomDigit = 0;
   String randomDigitStr = "";

   if (position.equalsIgnoreCase("year")) {
    int year = input.getYear() + 1900;
    String yearStr = String.valueOf(year);
    int newYear = year;
    while ((year == newYear)) {
     randomDigit = random.nextInt(10);
     randomDigitStr = String.valueOf(randomDigit);
     randomPosition = random.nextInt(yearStr.length());
     yearStr = yearStr.substring(0, randomPosition) + randomDigitStr + yearStr.substring(randomPosition + 1);
     newYear = Integer.parseInt(yearStr);
    }
    output.setYear(newYear - 1900);
    return output;
   }

   if (position.equalsIgnoreCase("month")) {
    int month = input.getMonth() + 1;
    int newMonth = 123;
    while ((newMonth > 12) || (month == newMonth)) {
     randomDigit = random.nextInt(10);
     randomDigitStr = String.valueOf(randomDigit);
     String monthStr = String.valueOf(month);
     randomPosition = random.nextInt(monthStr.length());
     monthStr = monthStr.substring(0, randomPosition) + randomDigitStr + monthStr.substring(randomPosition + 1);
     newMonth = Integer.parseInt(monthStr);
    }
    output.setMonth(newMonth - 1);
    return output;
   }

   if (position.equalsIgnoreCase("day")) {
    int newDay = 213;
    int day = input.getDate();
    while (newDay > getMaximumDaysOfMonth(input.getYear(),
      input.getMonth()) || (day == newDay)) {
     randomDigit = random.nextInt(10);
     randomDigitStr = String.valueOf(randomDigit);
     String dayStr = String.valueOf(day);
     randomPosition = random.nextInt(dayStr.length());
     dayStr = dayStr.substring(0, randomPosition) + randomDigitStr + dayStr.substring(randomPosition + 1);
     newDay = Integer.parseInt(dayStr);
    }
    output.setDate(newDay);
    return output;
   }

   return input;
  }
 };


 // Method 7: Invert characters in a field
 // Parameters:
 // 	Position: year - month - day
 public static final functionTwoParameters m7_invertCharacters = new functionTwoParameters() {

  @Override
  public Date run(Date input, String position) {
   nameOfLastExecutedFunction = "m7_invertCharacters";

   java.sql.Date output = (Date) input.clone();
   Random random = new Random();
   int randomPosition = 0;
   char[] chars;
   char temp = 0;
   if (position.equalsIgnoreCase("year")) {

    // Change it to string and invert
    int year = input.getYear() + 1900;
    String yearStr = String.valueOf(year);

    randomPosition = random.nextInt(yearStr.length() - 1);
    chars = yearStr.toCharArray();

    temp = chars[randomPosition];
    chars[randomPosition] = chars[randomPosition + 1];
    chars[randomPosition + 1] = temp;

    yearStr = new String(chars);

    output.setYear(Integer.parseInt(yearStr) - 1900);

    return output;
   }

   if (position.equalsIgnoreCase("month")) {
    int month = input.getMonth() + 1;
    String monthStr = String.valueOf(month);

    randomPosition = random.nextInt(monthStr.length() - 1);
    chars = monthStr.toCharArray();
    temp = 0;

    temp = chars[randomPosition];
    chars[randomPosition] = chars[randomPosition + 1];
    chars[randomPosition + 1] = temp;

    monthStr = new String(chars);
    output.setMonth(Integer.parseInt(monthStr) - 1);
    return output;
   }

   if (position.equalsIgnoreCase("day")) {
    int day = input.getDate();
    String dayStr = String.valueOf(day);

    randomPosition = random.nextInt(dayStr.length() - 1);
    chars = dayStr.toCharArray();

    temp = chars[randomPosition];
    chars[randomPosition] = chars[randomPosition + 1];
    chars[randomPosition + 1] = temp;

    dayStr = new String(chars);
    output.setDate(Integer.parseInt(dayStr));

    return output;
   }

   return input;
  }
 };


 // Method 8: Swap values among fields
 // Parameters:
 //	Source: year - month - day
 //  Target: year - month - day
 public static final functionThreeParameters m8_swapValues = new functionThreeParameters() {

  @Override
  public Date run(Date input, String source, String target) {
   nameOfLastExecutedFunction = "m8_swapValues";

   // receive source and target and change them
   java.sql.Date output = (Date) input.clone();

   int temp = 0;
   int year = input.getYear() + 1900;
   int month = input.getMonth() + 1;
   int day = input.getDate();

   if (source.equalsIgnoreCase("year") && target.equalsIgnoreCase("month")) {
    temp = year;
    year = month;
    month = temp;
    output.setYear(year - 1900);
    output.setMonth(month - 1);
    return output;
   }

   if (source.equalsIgnoreCase("year") && target.equalsIgnoreCase("day")) {
    temp = year;
    year = day;
    day = temp;
    output.setYear(year - 1900);
    output.setDate(day);
    return output;
   }

   if (source.equalsIgnoreCase("month") && target.equalsIgnoreCase("year")) {
    temp = month;
    month = year;
    year = temp;
    output.setMonth(month - 1);
    output.setYear(year - 1900);
    return output;
   }

   if (source.equalsIgnoreCase("month") && target.equalsIgnoreCase("day")) {
    temp = month;
    month = day;
    day = temp;
    output.setMonth(month - 1);
    output.setDate(day);
    return output;
   }

   if (source.equalsIgnoreCase("day") && target.equalsIgnoreCase("year")) {
    temp = day;
    day = year;
    year = temp;
    output.setDate(day);
    output.setYear(year - 1900);
    return output;
   }

   if (source.equalsIgnoreCase("day") && target.equalsIgnoreCase("month")) {
    temp = day;
    day = month;
    month = temp;
    output.setDate(day);
    output.setMonth(month - 1);
    return output;
   }
   return input;
  }
 };

 // Method 9: Replace with the default value
 // Parameters:
 //	defaultDate: the default date
 public static final functionTwoParameters3 m9_replaceWithDefault = new functionTwoParameters3() {

  @Override
  public Date run(Date input, Date defaultDate) {
   nameOfLastExecutedFunction = "m9_replaceWithDefault";

   return defaultDate;
  }
 };

 // Method 10: Remove first 2 digits of year field
 public static final functionOneParameter m10_removeDigitsFromYear = new functionOneParameter() {

  @Override
  public Date run(Date input) {
   nameOfLastExecutedFunction = "m10_removeDigitsFromYear";

   // year to string and after that substring and choose last 2
   java.sql.Date output = (Date) input.clone();
   int year = input.getYear() + 1900;
   String yearStr = String.valueOf(year);
   if (yearStr.length() > 1) {
    yearStr = yearStr.substring(2);
    output.setYear(Integer.parseInt(yearStr) - 1900);
   }
   return output;
  }
 };

 // Method 11: Add 100 years
 public static final functionOneParameter m11_add100years = new functionOneParameter() {

  @Override
  public Date run(Date input) {
   nameOfLastExecutedFunction = "m11_add100years";

   java.sql.Date output = (Date) input.clone();
   output.setYear(input.getYear() + 100);
   return output;
  }
 };

 // Method 12: Subtract 100 years
 public static final functionOneParameter m12_subtract100years = new functionOneParameter() {

  @Override
  public Date run(Date input) {
   nameOfLastExecutedFunction = "m12_subtract100years";

   java.sql.Date output = (Date) input.clone();
   output.setYear(input.getYear() - 100);
   return output;
  }
 };


 // Method 13: Duplicate
 // Parameters:
 //	Position: year - month - day 
 public static final functionTwoParameters m13_duplicate = new functionTwoParameters() {

  @Override
  public Date run(Date input, String position) {
   nameOfLastExecutedFunction = "m13_duplicate";

   java.sql.Date output = (Date) input.clone();
   Random random = new Random();

   int year = input.getYear() + 1900;
   int month = input.getMonth() + 1;
   int day = input.getDate();
   String yearStr = String.valueOf(year);
   String monthStr = String.valueOf(month);
   String dayStr = String.valueOf(day);
   int randomPosition = 0;

   if (position.equalsIgnoreCase("year")) {
    randomPosition = random.nextInt(yearStr.length());
    yearStr = yearStr.substring(0, randomPosition) + yearStr.charAt(randomPosition) + yearStr.substring(randomPosition);
    output.setYear(Integer.valueOf(yearStr) - 1900);
    return output;
   }
   if (position.equalsIgnoreCase("month")) {
    randomPosition = random.nextInt(monthStr.length());
    monthStr = monthStr.substring(0, randomPosition) + monthStr.charAt(randomPosition) + monthStr.substring(randomPosition);
    output.setMonth(Integer.valueOf(monthStr) - 1);
    return output;
   }
   if (position.equalsIgnoreCase("day")) {
    randomPosition = random.nextInt(dayStr.length());
    dayStr = dayStr.substring(0, randomPosition) + dayStr.charAt(randomPosition) + dayStr.substring(randomPosition);

    output.setDate(Integer.valueOf(dayStr));
    return output;
   }

   return input;
  }
 };


}