// Poor Data Injector
// 2015-2016 Seyma Nur Soydemir seyma@student.dei.uc.pt

package mutation;

import java.math.BigDecimal;
import java.util.Random;

public class Operators_Decimal {

 // Variable that holds the name of the last executed mutation
 public static String nameOfLastExecutedFunction = null;

 // Pre-defined values
 static String characterSet = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
 static String numericSet = "0123456789";
 static String symbolSet = "!2@£#§$%&/{[]}'?~^";

 // Interfaces for the various mutations
 public interface functionOneParameter {
  BigDecimal run(BigDecimal input);
 }

 public interface functionTwoParameters1 {
  BigDecimal run(BigDecimal input, String direction);
 }

 public interface functionTwoParameters2 {
  BigDecimal run(BigDecimal input, String position);
 }

 // Method 1: Replace by null value
 public static final functionOneParameter m1_replaceByNull = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {

   nameOfLastExecutedFunction = "m1_replaceByNull";

   return null;
  }
 };


 // Method 2: Replace by random same-size decimal value
 public static final functionOneParameter m2_replaceBySameSizeDecimal = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m2_replaceBySameSizeDecimal";

   String inputStr = String.valueOf(input);
   int radixLoc = inputStr.indexOf('.');
   int size = inputStr.length();

   Random random = new Random();
   String temp = "";

   for (int i = 0; i < size; i++) {
    int rand = -1;
    if (i == 0) {
     rand = random.nextInt(numericSet.length() - 1) + 1;
    } else {
     rand = random.nextInt(numericSet.length());
    }

    temp = temp + numericSet.charAt(rand);
   }
   if (radixLoc >= 0) {
    temp = temp.substring(0, radixLoc) + '.' + temp.substring(radixLoc + 1);
   }
   BigDecimal output = new BigDecimal(temp);
   return output;
  }
 };


 // Method 3: Add 1
 public static final functionOneParameter m3_add1 = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m3_add1";

   BigDecimal output = input;
   return output.add(new BigDecimal(1));
  }
 };

 // Method 4: Substract 1
 public static final functionOneParameter m4_substract1 = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m4_substract1";

   BigDecimal output = input;
   return output.subtract(new BigDecimal(1));
  }
 };


 // Method 5: Remove one random numeric character from data
 public static final functionOneParameter m5_removeOneNumeric = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m5_removeOneNumeric";

   BigDecimal temp = BigDecimal.ZERO;
   BigDecimal bg1 = new BigDecimal("-1");
   BigDecimal bg2 = new BigDecimal("0");

   if (input.compareTo(bg2) < 0) {
    temp = input;
    input = input.multiply(bg1);
   }

   Random random = new Random();

   String inputStr = String.valueOf(input);

   // We can only remove if there is more than 1 digit
   if (inputStr.length() > 1) {
    int randomPosition = random.nextInt(inputStr.length());
    while (inputStr.charAt(randomPosition) == '.') {
     randomPosition = random.nextInt(inputStr.length());
    }
    inputStr = inputStr.substring(0, randomPosition) + inputStr.substring(randomPosition + 1);
    input = new BigDecimal(inputStr);
   }

   if (temp.compareTo(bg2) < 0) {
    input = input.multiply(bg1);
   }
   return input;
  }
 };


 // Method 6:Duplicate one of the numeric characters
 // Parameters:
 // 	Position:	From 0 to the numeric value's original size. Repeated digit will be put in front of Position.
 public static final functionOneParameter m6_duplicateOneNumeric = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m6_duplicateOneNumeric";

   BigDecimal temp = BigDecimal.ZERO;

   BigDecimal bg1 = new BigDecimal("-1");
   BigDecimal bg2 = new BigDecimal("0");

   if (input.compareTo(bg2) < 0) {
    temp = input;
    input = input.multiply(bg1);
   }

   Random random = new Random();
   String inputStr = String.valueOf(input);
   int randomPosition = random.nextInt(inputStr.length());
   while (inputStr.charAt(randomPosition) == '.') {
    randomPosition = random.nextInt(inputStr.length());
   }
   inputStr = inputStr.substring(0, randomPosition) + inputStr.charAt(randomPosition) + inputStr.substring(randomPosition);
   input = new BigDecimal(inputStr);

   if (temp.compareTo(bg2) < 0) {
    input = input.multiply(bg1);
   }
   return input;
  }
 };

 // Method 7: Reverse two consecutive characters
 public static final functionOneParameter m7_invertConsecutiveDigits = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m7_invertConsecutiveDigits";

   BigDecimal temp = BigDecimal.ZERO;

   BigDecimal bg1 = new BigDecimal("-1");
   BigDecimal bg2 = new BigDecimal("0");

   if (input.compareTo(bg2) < 0) {
    temp = input;
    input = input.multiply(bg1);
   }

   Random random = new Random();
   char tempChar = 0;
   String inputStr = String.valueOf(input);

   if (inputStr.length() > 1) {
    int randomPosition = random.nextInt(inputStr.length() - 1);
    while (inputStr.charAt(randomPosition) == '.') {
     randomPosition = random.nextInt(inputStr.length() - 1);
    }
    char[] chars = inputStr.toCharArray();

    if (chars[randomPosition + 1] == '.') {
     chars[randomPosition + 1] = chars[randomPosition + 2];
    }
    tempChar = chars[randomPosition];
    chars[randomPosition] = chars[randomPosition + 1];
    chars[randomPosition + 1] = tempChar;

    inputStr = new String(chars);
    input = new BigDecimal(inputStr);
   }
   if (temp.compareTo(bg2) < 0) {
    input = input.multiply(bg1);
   }

   return input;
  }
 };


 // Method 8: Add random numeric character
 public static final functionOneParameter m8_addOneRandomNumeric = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m8_addOneRandomNumeric";

   BigDecimal temp = BigDecimal.ZERO;
   BigDecimal bg1 = new BigDecimal("-1");
   BigDecimal bg2 = new BigDecimal("0");

   if (input.compareTo(bg2) < 0) {
    temp = input;
    input = input.multiply(bg1);
   }

   Random random = new Random();
   String inputStr = String.valueOf(input);
   int randomDigit = random.nextInt(10);

   int randomPosition = random.nextInt(inputStr.length());
   while (inputStr.charAt(randomPosition) == '.') {
    randomPosition = random.nextInt(inputStr.length());
   }

   inputStr = inputStr.substring(0, randomPosition) + String.valueOf(randomDigit) + inputStr.substring(randomPosition);

   input = new BigDecimal(inputStr);

   if (temp.compareTo(bg2) < 0) {
    input = input.multiply(bg1);
   }

   return input;
  }
 };

 // Method 9: Flip sign
 public static final functionOneParameter m9_flipSign = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m9_flipSign";

   BigDecimal minus = new BigDecimal(-1);
   input = input.multiply(minus);

   return input;
  }
 };

 // Method 10: Zero out the integer part
 public static final functionOneParameter m10_zeroOutInteger = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m10_zeroOutInteger";

   BigDecimal temp = BigDecimal.ZERO;
   BigDecimal bg1 = new BigDecimal(-1);
   BigDecimal bg2 = new BigDecimal(0);

   if (input.compareTo(bg2) < 0) {
    temp = input;
    input = input.multiply(bg1);
   }

   Random random = new Random();
   String inputStr = String.valueOf(input);
   int radixLoc = inputStr.indexOf('.');
   String integerStr = inputStr.substring(0, radixLoc);

   inputStr = inputStr.replaceAll(integerStr, "0");
   input = new BigDecimal(inputStr);

   if (temp.compareTo(bg2) < 0) {
    input = input.multiply(bg1);
   }

   return input;

  }
 };

 // Method 11: Zero out the fractional part
 public static final functionOneParameter m11_zeroOutFractional = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m11_zeroOutFractional";

   BigDecimal temp = BigDecimal.ZERO;
   BigDecimal bg1 = new BigDecimal(-1);
   BigDecimal bg2 = new BigDecimal(0);

   if (input.compareTo(bg2) < 0) {
    temp = input;
    input = input.multiply(bg1);
   }

   Random random = new Random();
   String inputStr = String.valueOf(input);
   int radixLoc = inputStr.indexOf('.');
   String fractionalStr = inputStr.substring(radixLoc + 1);

   inputStr = inputStr.replaceAll(fractionalStr, "0");

   input = new BigDecimal(inputStr);
   if (temp.compareTo(bg2) < 0) {
    input = input.multiply(bg1);
   }

   return input;
  }
 };

 // Method 12: Remove the integer part
 public static final functionOneParameter m12_removeIntegerPart = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m12_removeIntegerPart";

   BigDecimal temp = BigDecimal.ZERO;
   BigDecimal bg1 = new BigDecimal(-1);
   BigDecimal bg2 = new BigDecimal(0);

   if (input.compareTo(bg2) < 0) {
    temp = input;
    input = input.multiply(bg1);
   }

   String inputStr = String.valueOf(input);
   int radixLoc = inputStr.indexOf('.');
   if (radixLoc < 0) {
    return BigDecimal.ZERO;
   }
   String fractionalStr = inputStr.substring(radixLoc + 1);

   inputStr = fractionalStr;
   input = new BigDecimal(inputStr);
   if (temp.compareTo(bg2) < 0) {
    input = input.multiply(bg1);
   }
   return input;
  }
 };

 // Method 13: Remove the fractional part
 public static final functionOneParameter m13_removeFractionalPart = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m13_removeFractionalPart";

   BigDecimal temp = BigDecimal.ZERO;
   BigDecimal bg1 = new BigDecimal(-1);
   BigDecimal bg2 = new BigDecimal(0);

   if (input.compareTo(bg2) < 0) {
    temp = input;
    input = input.multiply(bg1);
   }

   String inputStr = String.valueOf(input);
   int radixLoc = inputStr.indexOf('.');
   if (radixLoc < 0) {
    return input;
   }

   String integerStr = inputStr.substring(0, radixLoc);
   inputStr = integerStr;

   input = new BigDecimal(inputStr);
   if (temp.compareTo(bg2) < 0) {
    input = input.multiply(bg1);
   }
   return input;
  }
 };

 // Method 14: Remove the decimal separator
 public static final functionOneParameter m14_removeSeparator = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m14_removeSeparator";

   BigDecimal temp = BigDecimal.ZERO;
   BigDecimal bg1 = new BigDecimal(-1);
   BigDecimal bg2 = new BigDecimal(0);

   if (input.compareTo(bg2) < 0) {
    temp = input;
    input = input.multiply(bg1);
   }

   String inputStr = String.valueOf(input);
   int radixLoc = inputStr.indexOf('.');
   if (radixLoc < 0) {
    return input;
   }
   String fractionalStr = inputStr.substring(radixLoc + 1);
   String integerStr = inputStr.substring(0, radixLoc);

   inputStr = integerStr + fractionalStr;

   input = new BigDecimal(inputStr);
   if (temp.compareTo(bg2) < 0) {
    input = input.multiply(bg1);
   }
   return input;
  }
 };

 // Method 15: Add 10^-X
 public static final functionOneParameter m15_add10x = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m15_add10x";

   BigDecimal temp = BigDecimal.ZERO;
   BigDecimal bg1 = new BigDecimal(-1);
   BigDecimal bg2 = new BigDecimal(0);

   if (input.compareTo(bg2) < 0) {
    temp = input;
    input = input.multiply(bg1);
   }

   Random random = new Random();


   String inputStr = String.valueOf(input);
   int radixLoc = inputStr.indexOf('.');
   if (radixLoc < 0) {
    return BigDecimal.ZERO;
   }
   String fractionalStr = inputStr.substring(radixLoc + 1);
   String integerStr = inputStr.substring(0, radixLoc);

   int randomXPosition = random.nextInt(fractionalStr.length()) + 1;
   double addX = Math.pow(10, randomXPosition);
   BigDecimal add = new BigDecimal(addX);
   input = input.add(add);
   if (temp.compareTo(bg2) < 0) {
    input = input.multiply(bg1);
   }
   return input;
  }
 };

 // Method 16: Subtract 10^-X
 public static final functionOneParameter m16_substract10x = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m16_substract10x";

   BigDecimal temp = BigDecimal.ZERO;
   BigDecimal bg1 = new BigDecimal(-1);
   BigDecimal bg2 = new BigDecimal(0);

   if (input.compareTo(bg2) < 0) {
    temp = input;
    input = input.multiply(bg1);
   }

   Random random = new Random();


   String inputStr = String.valueOf(input);
   int radixLoc = inputStr.indexOf('.');
   if (radixLoc < 0) {
    return BigDecimal.ZERO;
   }
   String fractionalStr = inputStr.substring(radixLoc + 1);
   String integerStr = inputStr.substring(0, radixLoc);

   int randomXPosition = random.nextInt(fractionalStr.length()) + 1;
   double subX = Math.pow(10, randomXPosition);
   BigDecimal sub = new BigDecimal(subX);
   input = input.subtract(sub);
   if (temp.compareTo(bg2) < 0) {
    input = input.multiply(bg1);
   }
   return input;
  }
 };

 // Method 17: Shift decimal separator
 public static final functionTwoParameters1 m17_shiftDecimalSeparator = new functionTwoParameters1() {

  @Override
  public BigDecimal run(BigDecimal input, String direction) {
   nameOfLastExecutedFunction = "m17_shiftDecimalSeparator";

   Random random = new Random();
   BigDecimal temp = BigDecimal.ZERO;
   BigDecimal bg1 = new BigDecimal(-1);
   BigDecimal bg2 = new BigDecimal(0);

   if (input.compareTo(bg2) < 0) {
    temp = input;
    input = input.multiply(bg1);
   }

   String inputStr = String.valueOf(input);
   int radixLoc = inputStr.indexOf('.');
   if (radixLoc < 0) {
    return input;
   }

   String fractionalStr = inputStr.substring(radixLoc + 1);
   String integerStr = inputStr.substring(0, radixLoc);
   int length = 0;
   if (direction.equalsIgnoreCase("left")) {
    length = random.nextInt(integerStr.length());
    input.movePointLeft(length);
   }

   if (direction.equalsIgnoreCase("right")) {
    length = random.nextInt(fractionalStr.length());
    input.movePointRight(length);
   }
   if (temp.compareTo(bg2) < 0) {
    input = input.multiply(bg1);
   }
   return input;
  }
 };

 // Method 18: Add 1 zero next to decimal separator
 public static final functionTwoParameters2 m18_addZero = new functionTwoParameters2() {

  @Override
  public BigDecimal run(BigDecimal input, String position) {
   nameOfLastExecutedFunction = "m18_addZero";

   Random random = new Random();
   BigDecimal temp = BigDecimal.ZERO;
   BigDecimal bg1 = new BigDecimal(-1);
   BigDecimal bg2 = new BigDecimal(0);

   if (input.compareTo(bg2) < 0) {
    temp = input;
    input = input.multiply(bg1);
   }

   String inputStr = String.valueOf(input);
   int radixLoc = inputStr.indexOf('.');
   if (radixLoc < 0) {
    return input;
   }

   String fractionalStr = inputStr.substring(radixLoc + 1);
   String integerStr = inputStr.substring(0, radixLoc);

   if (position.equalsIgnoreCase("before")) {
    inputStr = integerStr + "0." + fractionalStr;
   }
   if (position.equalsIgnoreCase("after")) {
    inputStr = integerStr + ".0" + fractionalStr;

   }
   input = new BigDecimal(inputStr);
   if (temp.compareTo(bg2) < 0) {
    input = input.multiply(bg1);
   }
   return input;
  }
 };

 // Method 19: Swap fractional and integer parts
 public static final functionOneParameter m19_swapParts = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m19_swapParts";

   BigDecimal temp = BigDecimal.ZERO;
   BigDecimal bg1 = new BigDecimal("-1");
   BigDecimal bg2 = new BigDecimal("0");

   if (input.compareTo(bg2) < 0) {
    temp = input;
    input = input.multiply(bg1);
   }

   String inputStr = String.valueOf(input);
   int radixLoc = inputStr.indexOf('.');
   // if there is no .
   if (radixLoc == -1) {
    return input;
   }
   String fractionalStr = inputStr.substring(radixLoc + 1);
   String integerStr = inputStr.substring(0, radixLoc);

   inputStr = fractionalStr + '.' + integerStr;

   input = new BigDecimal(inputStr);
   if (temp.compareTo(bg2) < 0) {
    input = input.multiply(bg1);
   }

   return input;

  }
 };

 // Method 20: Reverse fractional part
 public static final functionOneParameter m20_reverseFractionalPart = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m20_reverseFractionalPart";

   BigDecimal temp = BigDecimal.ZERO;
   BigDecimal bg1 = new BigDecimal("-1");
   BigDecimal bg2 = new BigDecimal("0");

   if (input.compareTo(bg2) < 0) {
    temp = input;
    input = input.multiply(bg1);
   }

   String inputStr = String.valueOf(input);
   int radixLoc = inputStr.indexOf('.');
   // if there is no .
   if (radixLoc == -1) {
    return input;
   }
   String fractionalStr = inputStr.substring(radixLoc + 1);
   String integerStr = inputStr.substring(0, radixLoc);
   String reverseFractionalStr = new StringBuffer(fractionalStr).reverse().toString();


   inputStr = integerStr + '.' + reverseFractionalStr;
   input = new BigDecimal(inputStr);
   if (temp.compareTo(bg2) < 0) {
    input = input.multiply(bg1);
   }
   return input;
  }
 };


 // Method 21: Reverse integer part
 public static final functionOneParameter m21_reverseIntegerPart = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m21_reverseIntegerPart";

   BigDecimal temp = BigDecimal.ZERO;
   BigDecimal bg1 = new BigDecimal("-1");
   BigDecimal bg2 = new BigDecimal("0");

   if (input.compareTo(bg2) < 0) {
    temp = input;
    input = input.multiply(bg1);
   }

   String inputStr = String.valueOf(input);
   int radixLoc = inputStr.indexOf('.');
   // if there is no .
   if (radixLoc == -1) {
    return input;
   }
   String fractionalStr = inputStr.substring(radixLoc + 1);
   String integerStr = inputStr.substring(0, radixLoc);
   String reverseIntegerStr = new StringBuffer(integerStr).reverse().toString();

   inputStr = reverseIntegerStr + '.' + fractionalStr;
   input = new BigDecimal(inputStr);

   if (temp.compareTo(bg2) < 0) {
    input = input.multiply(bg1);
   }

   return input;
  }
 };


 // Method 22: Skipping decimal point near zero
 public static final functionOneParameter m22_skipPointNearZero = new functionOneParameter() {

  @Override
  public BigDecimal run(BigDecimal input) {
   nameOfLastExecutedFunction = "m22_skipPointNearZero";

   BigDecimal temp = BigDecimal.ZERO;
   BigDecimal bg1 = new BigDecimal("-1");
   BigDecimal bg2 = new BigDecimal("0");

   if (input.compareTo(bg2) < 0) {
    temp = input;
    input = input.multiply(bg1);
   }

   String inputStr = String.valueOf(input);

   int radixLoc = inputStr.indexOf('.');
   String integerStr = inputStr.substring(0, radixLoc);
   String fractionalStr = inputStr.substring(radixLoc + 1);


   if (integerStr.equalsIgnoreCase("0")) {
    inputStr = fractionalStr;
   }
   if (fractionalStr.equalsIgnoreCase("0")) {
    inputStr = integerStr;
   }

   input = new BigDecimal(inputStr);
   if (temp.compareTo(bg2) < 0) {
    input = input.multiply(bg1);
   }
   return input;
  }
 };

}