// Poor Data Injector
// 2015-2016 Seyma Nur Soydemir seyma@student.dei.uc.pt

package mutation;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import java.util.Scanner;

public class Operators_String {

 private static String getMethodName() {
  final StackTraceElement[] ste = Thread.currentThread().getStackTrace();
  return ste[ste.length - 1].getClassName();
 }

 // Variable that holds the name of the last executed mutation
 public static String nameOfLastExecutedFunction = null;

 // Interfaces for the various mutations
 public interface functionOneParameter {
  String run(String input);
 }

 public interface functionTwoParameterPosition {
  String run(String input, String position);
 }

 public interface functionTwoParameter2 {
  String run(String input, String position);
 }

 public interface functionTwoParameter3 {
  String run(String input, String position);
 }

 public interface functionTwoParameter4 {
  String run(String input, String position);
 }

 public interface functionTwoParameter5 {
  public String run(String input, String type);
 }

 public interface functionThreeParameter {
  String run(String input, String position, int quantity);
 }


 public interface functionThreeParameter2 {
  String run(String input, int begin, int length);
 }

 public interface functionFourParameter1 {
  String run(String input, int begin, int length, String position);
 }


 static ArrayList < String > nonprintable = new ArrayList < String > ();

 // Pre-defined variables
 static String characterSet = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
 static String numericSet = "0123456789";
 static String symbolSet = "!@£#§$€%&/{[]}'?~^";
 static String static_misspellings = "C://DEI//misspellings.txt";
 static String static_acronyms = "C://DEI//acronyms.txt";
 static String static_homonyms = "C://DEI//homonyms.txt";
 static String static_synonyms = "C://DEI//synonym.txt";

 // Initializes the list of all non-printable characters. Must be executed before nonprintable arrayList is used
 static void initializeNonprintableCharacters() {
  nonprintable.clear();

  // U+0000–U+001F
  for (int i = 0x0000; i <= 0x001F; i++) {
   nonprintable.add(Character.toString((char) i));
  }

  // U+0080–U+009F
  for (int i = 0x0080; i <= 0x009F; i++) {
   nonprintable.add(Character.toString((char) i));
  }

  nonprintable.add("\u007f");
 }


  // Method1: Replace by null value
  public static final functionOneParameter m1_replaceByNull = new functionOneParameter() {

   @Override
   public String run(String input) {
    nameOfLastExecutedFunction = "m1_replaceByNull";
    return null;
   }

  };


  // Method2 : Replace by empty string
  public static final functionOneParameter m2_replaceByEmptyString = new functionOneParameter() {

   @Override
   public String run(String input) {
    nameOfLastExecutedFunction = "m2_replaceByEmptyString";
    return "";
   }
  };

  // Method 3: Replace by same size string with non-printable characters
  public static final functionOneParameter m3_replaceByNonprintable = new functionOneParameter() {

   @Override
   public String run(String input) {
    nameOfLastExecutedFunction = "m3_replaceByNonprintable";
    initializeNonprintableCharacters();
    Random random = new Random();
    int randomNumber = random.nextInt(input.length());

    int randomNonPrintable = random.nextInt(nonprintable.size());
    String randomNonPrintableChar = nonprintable.get(randomNonPrintable);

    String output = null;
    output = input.substring(0, randomNumber) + randomNonPrintableChar + input.substring(randomNumber + 1);

    return output;

   }
  };


  // Method4: Add random nonprintable characters to string
  // Parameters:
  //	Position: prefix - middle - suffix
  public static final functionTwoParameterPosition m4_addRandomNonprintable = new functionTwoParameterPosition() {

   @Override
   public String run(String input, String position) {
    nameOfLastExecutedFunction = "m4_addRandomNonprintable";
    String output = null;
    Random random = new Random();
    int random_nonprintable_size = random.nextInt(input.length()) + 1;
    String subChar = "";

    initializeNonprintableCharacters();
    for (int i = 0; i < random_nonprintable_size; i++) {
     int random_nonprintable = random.nextInt(nonprintable.size());
     subChar = subChar + nonprintable.get(random_nonprintable);
    }

    if (position.equalsIgnoreCase("prefix")) {
     output = subChar + input;
    }

    if (position.equalsIgnoreCase("middle")) {
     int randomPosition = random.nextInt(input.length() - 1);
     output = input.substring(0, randomPosition + 1) + subChar + input.substring(randomPosition + 1);
    }

    if (position.equalsIgnoreCase("suffix")) {
     output = input + subChar;
    }

    return output;

   }
  };

  // Method 5: Replace by same size alphanumeric String
  public static final functionOneParameter m5_replaceWithAlphanumeric = new functionOneParameter() {

   @Override
   public String run(String input) {
    nameOfLastExecutedFunction = "m5_replaceWithAlphanumeric";
    Random random = new Random();
    String output = input;
    ArrayList < Integer > position_list = new ArrayList < Integer > ();

    // Number of characters we are going to replace
    int amount = random.nextInt(input.length()) + 1;

    // Which positions are we going to change?
    for (int i = 0; i < amount; i++) {
     int new_position = random.nextInt(input.length());
     while (position_list.contains(new_position)) {
      new_position = random.nextInt(input.length());
     }
     position_list.add(new_position);
    }

    for (int i: position_list) {
     int random_char_id = random.nextInt(characterSet.length());
     output = output.substring(0, i) + characterSet.charAt(random_char_id) + output.substring(i + 1);
    }

    return output;
   }
  };




  // Method 6: Add random numeric characters to string
  // Parameters:
  //	Position: prefix - middle - suffix
  public static final functionTwoParameterPosition m6_addRandomNumeric = new functionTwoParameterPosition() {

   @Override
   public String run(String input, String position) {
    nameOfLastExecutedFunction = "m6_addRandomNumeric";
    String output = "";
    Random random = new Random();
    int length = random.nextInt(input.length()) + 1;
    String newChar = "";

    for (int i = 0; i < length; i++) {

     int random_char_id = random.nextInt(numericSet.length());
     newChar = newChar + numericSet.charAt(random_char_id);
    }

    if (position.equalsIgnoreCase("prefix")) {
     output = newChar + input;
    }


    if (position.equalsIgnoreCase("middle")) {
     int randomPosition = random.nextInt(input.length());

     output = input.substring(0, randomPosition + 1) + newChar + input.substring(randomPosition + 1);

    }

    if (position.equalsIgnoreCase("suffix")) {
     output = input + newChar;
    }
    return output;
   }
  };


  // Method 7: Convert alphabetic characters to opposite case
  // Parameters:
  //	Position: First Character - Last Character - First character of each word - Last character of each word - Random position
  //			  Random position of each word
  public static final functionTwoParameter2 m7_convertOppositeCase = new functionTwoParameter2() {
   @Override
   public String run(String input, String position) {
    nameOfLastExecutedFunction = "m7_convertOppositeCase";
    String output = "";
    int length = 0;
    Random random = new Random();
    if (position.equalsIgnoreCase("first character")) {
     for (int begin = 0; begin < input.length(); begin++) {
      length = random.nextInt(input.length() - begin) + 1;
      output = input.substring(0, begin) + convertCase(input.substring(begin, begin + length)) + input.substring(begin + length);
      if (!input.equals(output)) {
       return output;
      }
     }
    }
    if (position.equalsIgnoreCase("last character")) {
     for (int begin = input.length(); begin >= 1; begin--) {
      length = random.nextInt(begin) + 1;
      output = input.substring(0, begin - length) + convertCase(input.substring(begin - length, begin)) + input.substring(begin);
      if (!input.equals(output)) {
       return output;
      }
     }
    }
    if (position.equalsIgnoreCase("first character of each word")) {
     String[] separated = input.split(" +");
     String modified = "";
     for (int i = 0; i < separated.length; i++) {
      modified = this.run(separated[i], "first character");
      if (i == 0) {
       output = modified;
      } else {
       output = output + " " + modified;
      }
     }
    }
    if (position.equalsIgnoreCase("last character of each word")) {
     String[] separated = input.split(" +");
     String modified = "";
     for (int i = 0; i < separated.length; i++) {
      modified = run(separated[i], "last character");
      if (i == 0) {
       output = modified;
      } else {
       output = output + " " + modified;
      }
     }
    }
    if (position.equalsIgnoreCase("random position")) {
     int randomPosition = random.nextInt(input.length());

     for (int begin = randomPosition; begin < input.length(); begin++) {
      length = random.nextInt(input.length() - begin) + 1;
      output = input.substring(0, begin) + convertCase(input.substring(begin, begin + length)) + input.substring(begin + length);
      if (!input.equals(output)) {
       return output;
      }
     }
     for (int begin = randomPosition; begin >= 1; begin--) {
      length = random.nextInt(begin) + 1;
      output = input.substring(0, begin - length) + convertCase(input.substring(begin - length, begin)) + input.substring(begin);
      if (!input.equals(output)) {
       return output;
      }
     }
    }
    if (position.equalsIgnoreCase("random position of each word")) {
     String[] separated = input.split(" +");
     String modified = "";
     for (int i = 0; i < separated.length; i++) {
      modified = run(separated[i], "random position");
      if (i == 0) {
       output = modified;
      } else {
       output = output + " " + modified;
      }
     }
    }
    return output;
   }
  };


  // Helper function that converts a string to opposite case
  public static String convertCase(String input) {

   String output = "";
   char[] chars = input.toCharArray();

   for (int i = 0; i < input.length(); i++) {
    if (Character.isUpperCase(chars[i])) {
     chars[i] = Character.toLowerCase(chars[i]);
    } else if (Character.isLowerCase(chars[i])) {
     chars[i] = Character.toUpperCase(chars[i]);
    }

   }

   output = new String(chars);

   return output;
  }


  // Method 8: Insert whitespace 
  // Parameters:
  //	Position: leading - trailing - random between words
  //	Quantity: integer with amount of whitesoaces to add
  public static final functionThreeParameter m8_insertWhitespace = new functionThreeParameter() {
   @Override
   public String run(String input, String position, int quantity) {
    nameOfLastExecutedFunction = "m8_insertWhitespace";
    String output = "";
    String extra_whitespaces = "";
    for (int i = 0; i < quantity; i++) {
     extra_whitespaces = extra_whitespaces + " ";
    }

    if (position.equalsIgnoreCase("leading")) {
     output = extra_whitespaces + input;
    }

    if (position.equalsIgnoreCase("trailing")) {
     output = input + extra_whitespaces;
    }

    if (position.equalsIgnoreCase("random between words")) {
     String[] words = input.split(" +");
     String word = "";

     for (int i = 0; i < words.length; i++) {
      if (i == 0) {
       output = words[i];
      } else {
       output = output + extra_whitespaces + " " + words[i];
      }
     }
    }

    return output;
   }
  };


  // Method 9: Add characters to string: 
  // Parameters:
  //	Position: prefix - middle - suffix
  // 	Quantity: Random between 1 and the string's original size. The new characters to add must be present in original string.
  public static final functionTwoParameterPosition m9_addCharactersToString = new functionTwoParameterPosition() {

   @Override
   public String run(String input, String position) {
    nameOfLastExecutedFunction = "m9_addCharactersToString";
    String output = "";
    Random random = new Random();
    int length = random.nextInt(input.length()) + 1;
    String newChar = "";

    for (int i = 0; i < length; i++) {

     int random_char_id = random.nextInt(input.length());
     newChar = newChar + input.charAt(random_char_id);
    }

    if (position.equalsIgnoreCase("prefix")) {
     output = newChar + input;
    }


    if (position.equalsIgnoreCase("middle")) {
     int randomPosition = random.nextInt(input.length() - 1);

     output = input.substring(0, randomPosition + 1) + newChar + input.substring(randomPosition + 1);

    }

    if (position.equalsIgnoreCase("suffix")) {
     output = input + newChar;
    }
    return output;
   }
  };


  // Method 10: Add substring to string
  // Parameters:
  //	Begin:	From 0 to the string's original size minus 1.
  // 	Length: From 1 to the string's original size minus Begin
  // 	Position: Prefix - Middle - Suffix
  public static final functionFourParameter1 m10_addSubstring = new functionFourParameter1() {

   @Override
   public String run(String input, int begin, int length, String position) {
    nameOfLastExecutedFunction = "m10_addSubstring";
    String output = "";
    Random random = new Random();
    String substring = input.substring(begin, begin + length);

    if (position.equalsIgnoreCase("prefix")) {
     output = substring + input;
    }

    if (position.equalsIgnoreCase("middle")) {
     int randomPosition = random.nextInt(input.length() - 1);
     output = input.substring(0, randomPosition + 1) + substring + input.substring(randomPosition + 1);
    }

    if (position.equalsIgnoreCase("suffix")) {
     output = input + substring;
    }
    return output;
   }
  };

  // Method 11: Move substring 
  // Parameters:
  // 	Begin:	From 0 to the string's original size minus 1.
  // 	Length:	From 1 to the string's original size minus Begin.
  //	Position:	Prefix - Middle - Suffix
  public static final functionFourParameter1 m11_moveSubstring = new functionFourParameter1() {

   @Override
   public String run(String input, int begin, int length, String position) {
    nameOfLastExecutedFunction = "m11_moveSubstring";
    String output = "";
    Random random = new Random();
    String substring = input.substring(begin, begin + length);

    if (position.equalsIgnoreCase("prefix")) {
     output = substring + input.substring(0, begin) + input.substring(begin + length);
    }

    if (position.equalsIgnoreCase("middle")) {
     String newString = input.substring(0, begin) + input.substring(begin + length);
     int randomPosition = random.nextInt(newString.length());
     while ((newString.length() > 1) && (randomPosition == begin)) {
      randomPosition = random.nextInt(newString.length());
     }
     output = newString.substring(0, randomPosition + 1) + substring + newString.substring(randomPosition + 1);
    }

    if (position.equalsIgnoreCase("suffix")) {
     output = input.substring(0, begin) + input.substring(begin + length) + substring;
    }
    return output;
   }
  };

  // Method 12: Remove substring
  // Parameters:
  // 	Begin:	From 0 to the string's original size minus 1.
  // 	Length:	From 1 to the string's original size minus Begin.
  public static final functionThreeParameter2 m12_removeSubstring = new functionThreeParameter2() {

   @Override
   public String run(String input, int begin, int length) {
    nameOfLastExecutedFunction = "m12_removeSubstring";
    String output = "";

    output = input.substring(0, begin) + input.substring(begin + length);

    return output;
   }
  };



  // Method 13: Remove whitespace
  // Parameters:
  // 	Position: leading - trailing - random between words
  public static final functionTwoParameter3 m13_removeWhitespace = new functionTwoParameter3() {

   @Override
   public String run(String input, String position) {
    nameOfLastExecutedFunction = "m13_removeWhitespace";

    String output = "";
    Random random = new Random();

    if (position.equalsIgnoreCase("leading")) {
     int firstNonWhitespace = input.length();
     for (int i = 0; i < input.length(); i++) {
      if (input.charAt(i) != ' ') {
       firstNonWhitespace = i;
       break;
      }
     }
     output = input.substring(firstNonWhitespace);
    }

    if (position.equalsIgnoreCase("trailing")) {
     int lastNonWhitespace = input.length();
     for (int i = 0; i < input.length(); i++) {
      if (input.charAt(i) != ' ') {
       lastNonWhitespace = i;
      }
     }
     output = input.substring(0, lastNonWhitespace + 1);
    }

    if (position.equalsIgnoreCase("random between words")) {
     String[] words = input.split(" +");
     int randomWordThatWillLoseWhitespace = random.nextInt(words.length);
     for (int i = 0; i < words.length; i++) {
      if (i == randomWordThatWillLoseWhitespace) {
       // In this word we dont put any whitespace
       output = output + words[i];
      } else {
       // Do as usual
       if (i == 0) {
        output = words[i];
       } else {
        output = output + " " + words[i];
       }
      }
     }
    }

    return output;
   }
  };


  // Method 14: Delimited reverse
  // Parameters:
  // 	Begin:	From 0 to the string's original size minus 2.
  // 	Length:	From 2 to the string's original size minus Begin
  public static final functionThreeParameter2 m14_delimitedReverse = new functionThreeParameter2() {

   @Override
   public String run(String input, int begin, int length) {
    nameOfLastExecutedFunction = "m14_delimitedReverse";
    String output = "";

    String substring = input.substring(begin, begin + length);

    String reverse = new StringBuffer(substring).reverse().toString();

    output = input.substring(0, begin) + reverse + input.substring(begin + length);

    return output;
   }
  };


  // Method 15: Replace by predefined SQL string
  public static final functionOneParameter m15_replaceBySQLString = new functionOneParameter() {

   @Override
   public String run(String input) {
    nameOfLastExecutedFunction = "m15_replaceBySQLString";
    // Predefined keywords to replace
    String[] sqlKeywords = {
     "INSERT",
     "SELECT",
     "*",
     "FROM",
     "UPDATE",
     "DELETE",
     "WHERE",
     "VALUES",
     "INPUT",
     "AND",
     "SET"
    };
    String output = "";
    Random random = new Random();
    int randomSQL = random.nextInt(sqlKeywords.length);

    output = sqlKeywords[randomSQL];
    return output;
   }
  };

  // Method 16:	Replace with symbols
  public static final functionOneParameter m16_replaceAllBySymbol = new functionOneParameter() {

   @Override
   public String run(String input) {
    nameOfLastExecutedFunction = "m16_replaceAllBySymbol";
    String output = "";
    Random random = new Random();
    int randomSymbol = random.nextInt(symbolSet.length());

    output = "" + symbolSet.charAt(randomSymbol);
    return output;
   }
  };

  // Method 17: Misspelling
  // Replace one word by a misspelled word (Dictionary-based) or in case of no match being found, use a random single edit operation
  // (insertion; deletion; substitution of a single character or transposition of two adjacent characters) over a randomly selected word
  public static final functionOneParameter m17_misspeling = new functionOneParameter() {

   @Override
   public String run(String input) {
    nameOfLastExecutedFunction = "m17_misspeling";
    // Create the dictionary
    Map < String, String > dictionary = new HashMap < String, String > ();
    String line = "";
    String[] words2;
    Scanner s;

    try {
     s = new Scanner(new File(static_misspellings));
     while (s.hasNextLine()) {
      line = s.nextLine();

      words2 = line.split(" ");
      dictionary.put(words2[0], words2[1]);
     }
     s.close();
    } catch (FileNotFoundException e) {
     e.printStackTrace();
    }



    String output = "";
    String[] words = input.split(" +");

    Random random = new Random();
    int randomWord = random.nextInt(words.length);

    // Check in dictionary
    if (dictionary.containsKey(words[randomWord])) {
     // Replace by entry in dictionary
     words[randomWord] = dictionary.get(words[randomWord]);
    } else {
     // Random single edit operation (insert, delete, substitute)
     int randomPositionInWord = random.nextInt(words[randomWord].length());
     int randomOperation = -1;
     if (words[randomWord].length() <= 1) {
      randomOperation = random.nextInt(3);

     } else {
      randomOperation = random.nextInt(4);
     }
     char randomNewCharacter = characterSet.charAt(random.nextInt(characterSet.length()));
     if (randomOperation == 0) // Insert
     {
      words[randomWord] = words[randomWord].substring(0, randomPositionInWord) + randomNewCharacter + words[randomWord].substring(randomPositionInWord);
     }
     if (randomOperation == 1) // Delete
     {
      words[randomWord] = words[randomWord].substring(0, randomPositionInWord) + words[randomWord].substring(randomPositionInWord + 1);
     }
     if (randomOperation == 2) // Substitute
     {
      words[randomWord] = words[randomWord].substring(0, randomPositionInWord) + randomNewCharacter + words[randomWord].substring(randomPositionInWord + 1);
     }
     if (randomOperation == 3) // Transposition of two adjacent characters
     {
      if (randomPositionInWord == (words[randomWord].length() - 1)) {
       randomPositionInWord--;
      }
      char[] chars = words[randomWord].toCharArray();
      char temp = 's';
      temp = chars[randomPositionInWord];
      chars[randomPositionInWord] = chars[randomPositionInWord + 1];
      chars[randomPositionInWord + 1] = temp;
      words[randomWord] = String.valueOf(chars);
     }
    }

    // Rebuild the output string
    for (int i = 0; i < words.length; i++) {
     if (i == 0) {
      output = words[i];
     } else {
      output = output + " " + words[i];
     }
    }
    return output;
   }
  };


  // Helper function to create a random string
  static String createRandomString(String input, int max_size) {
   String output = "";
   Random random = new Random();
   for (int i = 0; i < max_size; i++) {
    char extra = characterSet.charAt(random.nextInt(characterSet.length()));
    output = output + extra;
   }
   return output;
  }

  // Method 18: Replace with a limit size
  // Parameters:
  // 	Type: UTF maximum size, UTF maximum size +1
  public static final functionTwoParameter5 m18_replaceLimitSize = new functionTwoParameter5() {

   @Override
   public String run(String input, String type) {
    nameOfLastExecutedFunction = "m18_replaceLimitSize";

    String output = "";

    if (type.equalsIgnoreCase("UTF maximum")) // Internet resources say that max is 65534
    {
     output = createRandomString(input, 65534);

    }
    if (type.equalsIgnoreCase("UTF maximum + 1")) {
     output = createRandomString(input, 65535);

    }
    return output;
   }
  };

  // Method 19: Replace with an imprecise value	 -
  // Dictionary based: Chooses a single random word and replaces it by the respective acronym or abbreviation
  public static final functionOneParameter m19_replaceByAcronymn = new functionOneParameter() {

   @Override
   public String run(String input) {
    nameOfLastExecutedFunction = "m19_replacebyAcronym";

    String output = "";
    // Create the dictionary
    Map < String, String > dictionary = new HashMap < String, String > ();
    String line = "";
    String[] words;
    Scanner s;
    // ("\\W+") == not alphanumeric character
    String[] inputSeparated = input.split(" +");

    try {
     s = new Scanner(new File(static_acronyms));
     while (s.hasNextLine()) {
      line = s.nextLine();

      words = line.split(":");
      dictionary.put(words[0], words[1]);
     }
     s.close();
    } catch (FileNotFoundException e) {
     e.printStackTrace();
    }

    Random random = new Random();
    String w = "";
    for (int i = 0; i < inputSeparated.length; i++) {
     if (dictionary.containsKey(inputSeparated[i])) {
      inputSeparated[i] = dictionary.get(inputSeparated[i]);
     }

     if (i == 0) {
      output = inputSeparated[i];
     } else {
      output = output + " " + inputSeparated[i];
     }

    }

    return output;
   }
  };

  // Method 20: Replace by specific type string
  // Use all faults for other data types (e.g., numeric, date, time, timestamp, boolean mutations, to be defined)
  public static final functionOneParameter m20_replaceBySpecificTypeString = new functionOneParameter() {

   @Override
   public String run(String input) {
    nameOfLastExecutedFunction = "m20_replaceBySpecificTypeString";

    Random r = new Random();
    int new_type = r.nextInt(6);
    if (new_type == 0) {
     // Number
     // Get a random number between 0 and Integer.MAX and convert to string
     int new_number = r.nextInt(Integer.MAX_VALUE);
     return String.valueOf(new_number);
    }
    if (new_type == 1) {
     // Floating
     // Get a random float  between 0 and FLoat.MAX_VALUE and convert to string
     float new_number = r.nextFloat() * Float.MAX_VALUE;
     return String.valueOf(new_number);
    }
    if (new_type == 2) {
     // Boolean
     String[] predefined_list = {
      "true",
      "false"
     };
     int new_number = r.nextInt(predefined_list.length);
     return predefined_list[new_number];
    }
    if (new_type == 3) {
     // Date
     int year = r.nextInt(2000);
     int month = r.nextInt(11);
     int date = r.nextInt(31);
     java.sql.Date tmp_date = new java.sql.Date(0);
     tmp_date.setYear(year);
     tmp_date.setMonth(month);
     tmp_date.setDate(date);

     return tmp_date.toString();
    }
    if (new_type == 4) {
     // Time
     int hour = r.nextInt(24);
     int minute = r.nextInt(60);
     int second = r.nextInt(60);
     java.sql.Time tmp_time = new java.sql.Time(0);
     tmp_time.setHours(hour);
     tmp_time.setMinutes(minute);
     tmp_time.setSeconds(second);

     return tmp_time.toString();
    }
    if (new_type == 5) {
     // Timestamp
     int year = r.nextInt(2000);
     int month = r.nextInt(11);
     int date = r.nextInt(31);
     int hour = r.nextInt(24);
     int minute = r.nextInt(60);
     int second = r.nextInt(60);
     int nanos = r.nextInt(1000000000);

     java.sql.Timestamp tmp_timestamp = new java.sql.Timestamp(0);
     tmp_timestamp.setYear(year);
     tmp_timestamp.setMonth(month);
     tmp_timestamp.setDate(date);
     tmp_timestamp.setHours(hour);
     tmp_timestamp.setMinutes(minute);
     tmp_timestamp.setSeconds(second);
     tmp_timestamp.setNanos(nanos);
     return tmp_timestamp.toString();
    }
    return input;
   }
  };

  // Method 21: Replace whitespaces by symbols 
  // Replaces all the white spaces by symbols (_, . , %, $, €)
  public static final functionOneParameter m21_replaceWhitespaceBySymbols = new functionOneParameter() {

   @Override
   public String run(String input) {
    nameOfLastExecutedFunction = "m21_replaceWhitespaceBySymbols";
    Random random = new Random();
    String output = "";

    char[] charArr = input.toCharArray();

    for (int i = 0; i < charArr.length; i++) {
     if (charArr[i] == ' ') {
      int random_char_id = random.nextInt(symbolSet.length());

      charArr[i] = symbolSet.charAt(random_char_id);
     }
    }
    output = String.valueOf(charArr);

    return output;
   }
  };

  // Method 22: Add extraneous data
  // Parameters:
  //	Position: after - before
  public static final functionTwoParameter4 m22_addExtraneous = new functionTwoParameter4() {

   @Override
   public String run(String input, String position) {
    nameOfLastExecutedFunction = "m22_addExtraneous";
    if (input == null) {
     return null;
    }
    String output = "";
    String[] after = {
     "Director",
     "CEO",
     "Student"
    };
    String[] before = {
     "PhD",
     "Professor",
     "Prof.",
     "Dt.",
     "Mr.",
     "Mrs.",
     "Eng.",
     "Doctor"
    };
    Random random = new Random();
    int randomWord = 0;
    if (position.equalsIgnoreCase("after")) {
     randomWord = random.nextInt(after.length);

     output = input + after[randomWord];
    }

    if (position.equalsIgnoreCase("before")) {
     randomWord = random.nextInt(before.length);
     output = before[randomWord] + input;
    }
    return output;
   }
  };


  // Method23: Replace by Homonym
  public static final functionOneParameter m23_replaceByHomonyms = new functionOneParameter() {

   @Override
   public String run(String input) {
    nameOfLastExecutedFunction = "m23_replaceByHomonyms";
    String output = "";
    // Create the dictionary
    Map < String, ArrayList < String >> dictionary = new HashMap < String, ArrayList < String >> ();
    String line = "";
    String[] words;
    Scanner s;

    String[] inputSeparated = input.split(" +");
    try {
     s = new Scanner(new File(static_homonyms));
     ArrayList < String > list = new ArrayList < String > ();
     while (s.hasNext()) {
      line = s.next();
      words = line.split(",");
      ArrayList < String > homonyms = new ArrayList < String > ();

      for (int i = 1; i < words.length; i++) {
       homonyms.add(words[i]);
      }
      dictionary.put(words[0], homonyms);

      list.add(line);
     }
     s.close();
    } catch (FileNotFoundException e) {
     e.printStackTrace();
    }

    Random random = new Random();

    ArrayList < String > homonymWord = new ArrayList < String > ();
    String w = "";
    for (int i = 0; i < inputSeparated.length; i++) {
     if (dictionary.containsKey(inputSeparated[i])) {
      homonymWord = dictionary.get(inputSeparated[i]);
      inputSeparated[i] = homonymWord.get(random.nextInt(homonymWord.size()));
     }

     if (i == 0) {
      output = inputSeparated[i];
     } else {
      output = output + " " + inputSeparated[i];
     }
    }
    return output;
   }
  };


  // Method 24: Synonym
  public static final functionOneParameter m24_replaceBySynonyms = new functionOneParameter() {

   @Override
   public String run(String input) {
    nameOfLastExecutedFunction = "m24_replaceBySynonyms";
    String output = "";
    // Create the dictionary
    Map < String, ArrayList < String >> dictionary = new HashMap < String, ArrayList < String >> ();
    String line = "";
    String[] words;
    Scanner s;

    String[] inputSeparated = input.split(" +");
    try {
     s = new Scanner(new File(static_synonyms));
     ArrayList < String > list = new ArrayList < String > ();
     while (s.hasNext()) {
      line = s.next();
      words = line.split(",");
      ArrayList < String > synonym = new ArrayList < String > ();

      for (int i = 1; i < words.length; i++) {
       synonym.add(words[i]);
      }
      dictionary.put(words[0], synonym);

      list.add(line);
     }
     s.close();
    } catch (FileNotFoundException e) {
     // TODO Auto-generated catch block
     e.printStackTrace();
    }

    Random random = new Random();
    ArrayList < String > synonymWord = new ArrayList < String > ();
    String w = "";
    for (int i = 0; i < inputSeparated.length; i++) {
     if (dictionary.containsKey(inputSeparated[i])) {
      synonymWord = dictionary.get(inputSeparated[i]);
      inputSeparated[i] = synonymWord.get(random.nextInt(synonymWord.size()));
     }

     if (i == 0) {
      output = inputSeparated[i];
     } else {
      output = output + " " + inputSeparated[i];
     }
    }
    return output;
   }
  };

  // Helper function that changes ordering of words randomly
  public static String changeOrdering(String input) {
   String output = "";
   String[] words = input.split(" +");
   String temp = "";

   Random random = new Random();
   int r = random.nextInt(words.length - 1);

   temp = words[r];
   words[r] = words[r + 1];
   words[r + 1] = temp;
   for (int i = 0; i < words.length; i++) {
    if (i == 0) {
     output = words[i];
    } else {
     output = output + " " + words[i];
    }
   }

   return output;

  }



 }