// Poor Data Injector
// 2015-2016 Seyma Nur Soydemir seyma@student.dei.uc.pt

package mutation;

import java.sql.Timestamp;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Random;

public class Operators_Timestamp {

 // Variable that holds the name of the last executed mutation
 public static String nameOfLastExecutedFunction = null;


 // Interfaces for the various mutations
 public interface functionOneParameter {
  java.sql.Timestamp run(java.sql.Timestamp input);
 }

 public interface functionTwoParameters {
  java.sql.Timestamp run(java.sql.Timestamp input, String field);
 }

 public interface functionTwoParameters2 {
  java.sql.Timestamp run(java.sql.Timestamp input, String position);
 }

 // Static definitions for the limits
 static int maximum_allowed_year_field = 9999;
 static int minimum_allowed_year_field = -1900;

 static int maximum_allowed_month_field = 11;
 static int minimum_allowed_month_field = 0;

 static int maximum_allowed_day_field = 31;
 static int minimum_allowed_day_field = 1;

 // Obtain maximum days in a month of a certain year
 public static int getMaximumDaysOfMonth(int year, int month) {
  // Date's year 0 == 1900, but GregorianCalendar's year 0 == 1700
  Calendar mycal = new GregorianCalendar(year + (1900 - 1700), month, 1);

  return mycal.getActualMaximum(Calendar.DAY_OF_MONTH); // 28
 }

 // Method 1: Replace by null timestamp
 public static final functionOneParameter m1_replaceByNull = new functionOneParameter() {

  @Override
  public Timestamp run(Timestamp input) {
   nameOfLastExecutedFunction = "m1_replaceByNull";
   return null;
  }
 };

 // Method 2: Replace by empty dummy timestamp
 public static final functionOneParameter m2_replaceByEmptyDummy = new functionOneParameter() {

  @Override
  public Timestamp run(Timestamp input) {
   nameOfLastExecutedFunction = "m2_replaceByEmptyDummy";

   java.sql.Timestamp output = new Timestamp(0, 0, 0, 0, 0, 0, 0);
   return output;
  }
 };

 // Method 3: Replace by random timestamp
 // Parameters:
 // 	Position: Year - Month - Day - Hour - Minute - Second - Nanosecond
 // 	Value:    Uniformly random between the maximum and minimum allowed value for the field
 public static final functionTwoParameters m3_replaceByRandom = new functionTwoParameters() {

  @Override
  public Timestamp run(Timestamp input, String field) {
   nameOfLastExecutedFunction = "m3_replaceByRandom";

   java.sql.Timestamp output = (Timestamp) input.clone();

   Random random = new Random();

   if (field.equalsIgnoreCase("year")) {
    int randomYear = random.nextInt(9999);
    output.setYear(randomYear);
   }

   if (field.equalsIgnoreCase("month")) {
    int randomMonth = random.nextInt(12);
    output.setMonth(randomMonth - 1);
   }

   if (field.equalsIgnoreCase("day")) {
    int maxDayForMonthAndYear = getMaximumDaysOfMonth(input.getYear(),
     input.getMonth());
    int randomDay = random.nextInt(maxDayForMonthAndYear) + 1;
    output.setDate(randomDay);
   }

   if (field.equalsIgnoreCase("hour")) {
    int randomHour = random.nextInt(24);
    output.setHours(randomHour);
   }

   if (field.equalsIgnoreCase("minute")) {
    int randomMinute = random.nextInt(60);
    output.setMinutes(randomMinute);
   }

   if (field.equalsIgnoreCase("second")) {
    int randomSecond = random.nextInt(60);
    output.setSeconds(randomSecond);
   }

   if (field.equalsIgnoreCase("nano")) {
    int randomNano = random.nextInt(999999999 + 1);
    output.setNanos(randomNano);
   }
   return output;
  }
 };

 // Method 8: Reverse field 
 // Parameters:
 // 	Field: Year - Month - Day - Hour - Minute - Second - Nanosecond
 public static final functionTwoParameters m4_reverseField = new functionTwoParameters() {

  @Override
  public Timestamp run(Timestamp input, String field) {
   nameOfLastExecutedFunction = "m4_reverseField";

   java.sql.Timestamp output = (Timestamp) input.clone();

   Random random = new Random();
   int randomPosition = 0;
   char[] chars;
   char temp = 0;

   if (field.equalsIgnoreCase("year")) {
    // convert to chars or substring

    // Change it to string and invert
    int year = input.getYear() + 1900;
    String yearStr = String.valueOf(year);

    randomPosition = random.nextInt(yearStr.length() - 1);
    chars = yearStr.toCharArray();

    temp = chars[randomPosition];
    chars[randomPosition] = chars[randomPosition + 1];
    chars[randomPosition + 1] = temp;

    yearStr = new String(chars);

    output.setYear(Integer.parseInt(yearStr) - 1900);

    return output;
   }

   if (field.equalsIgnoreCase("month")) {
    int month = input.getMonth() + 1;
    String monthStr = String.valueOf(month);

    if (monthStr.length() < 2) {
     monthStr = "0" + monthStr;
    }

    chars = monthStr.toCharArray();
    temp = 0;

    temp = chars[0];
    chars[0] = chars[1];
    chars[1] = temp;

    monthStr = new String(chars);
    output.setMonth(Integer.parseInt(monthStr) - 1);

    return output;
   }

   if (field.equalsIgnoreCase("day")) {
    int day = input.getDate();
    String dayStr = String.valueOf(day);
    if (dayStr.length() < 2) {
     dayStr = "0" + dayStr;
    }

    chars = dayStr.toCharArray();

    temp = chars[0];
    chars[0] = chars[1];
    chars[1] = temp;

    dayStr = new String(chars);
    output.setDate(Integer.parseInt(dayStr));

    return output;
   }

   if (field.equalsIgnoreCase("hour")) {
    // convert to chars or substring

    // Change it to string and invert
    int hour = input.getHours();
    String hourStr = String.valueOf(hour);

    if (hourStr.length() < 2) {
     hourStr = "0" + hourStr;
    }
    chars = hourStr.toCharArray();

    temp = chars[0];
    chars[0] = chars[1];
    chars[1] = temp;

    hourStr = new String(chars);

    output.setHours(Integer.parseInt(hourStr));

    return output;
   }

   if (field.equalsIgnoreCase("minute")) {
    int minute = input.getMinutes();
    String minuteStr = String.valueOf(minute);
    if (minuteStr.length() < 2) {
     minuteStr = "0" + minuteStr;
    }

    chars = minuteStr.toCharArray();
    temp = 0;

    temp = chars[0];
    chars[0] = chars[1];
    chars[1] = temp;

    minuteStr = new String(chars);
    output.setMinutes(Integer.parseInt(minuteStr));

    return output;
   }

   if (field.equalsIgnoreCase("second")) {
    int second = input.getSeconds();
    String secondStr = String.valueOf(second);

    if (secondStr.length() < 2) {
     secondStr = "0" + secondStr;
    }

    chars = secondStr.toCharArray();

    temp = chars[0];
    chars[0] = chars[1];
    chars[1] = temp;

    secondStr = new String(chars);
    output.setSeconds(Integer.parseInt(secondStr));

    return output;
   }

   if (field.equalsIgnoreCase("nano")) {
    int nano = input.getNanos();
    String nanoStr = String.valueOf(nano);

    randomPosition = random.nextInt(nanoStr.length() - 1);
    chars = nanoStr.toCharArray();

    temp = chars[randomPosition];
    chars[randomPosition] = chars[randomPosition + 1];
    chars[randomPosition + 1] = temp;

    nanoStr = new String(chars);

    output.setNanos(Integer.parseInt(nanoStr));

    return output;
   }
   return input;
  }
 };

 // Method 9: Add one extra character to timestamp
 // Parameters:
 // 	Position: Before year value After year - Before month - After month value
 // - Before day - After day - Before hour - After hour - Before minute - After minute 
 // - Before second - After second - Before nano - After nano

 public static final functionTwoParameters2 m5_addOneExtraChar = new functionTwoParameters2() {

  @Override
  public Timestamp run(Timestamp input, String position) {
   nameOfLastExecutedFunction = "m5_addOneExtraChar";

   java.sql.Timestamp output = (Timestamp) input.clone();

   Random random = new Random();
   // java.sql.Date output = (Date) input.clone();
   int randomDigit = 0;
   // java.sql.Date output = input;
   int hour = input.getHours();
   int minute = input.getMinutes();
   int second = input.getSeconds();
   int nano = input.getNanos();
   String hourStr = String.valueOf(hour);
   String minuteStr = String.valueOf(minute);
   String secondStr = String.valueOf(second);
   String nanoStr = String.valueOf(nano);
   if (position.equalsIgnoreCase("Before year")) {
    int currentYear = input.getYear() + 1900;
    String currentYearStr = String.valueOf(currentYear);
    int newYear = currentYear;
    while ((currentYear == newYear)) {
     randomDigit = random.nextInt(10);
     String randomStr = String.valueOf(randomDigit);
     String temp = randomStr + currentYearStr;
     newYear = Integer.parseInt(temp);
    }
    output.setYear(newYear - 1900);
    return output;
   }

   if (position.equalsIgnoreCase("After year")) {
    int currentYear = input.getYear() + 1900;
    String currentYearStr = String.valueOf(currentYear);
    int newYear = currentYear;
    while ((currentYear == newYear)) {
     randomDigit = random.nextInt(10);
     String randomStr = String.valueOf(randomDigit);
     String temp = currentYearStr + randomStr;
     newYear = Integer.parseInt(temp);
    }
    output.setYear(newYear - 1900);
    return output;
   }

   if (position.equalsIgnoreCase("Before month")) {
    int currentMonth = input.getMonth() + 1;
    int newMonth = -1;
    String currentMonthStr = String.valueOf(currentMonth);
    randomDigit = random.nextInt(9) + 1;
    String randomStr = String.valueOf(randomDigit);
    String temp = randomStr + currentMonthStr;
    newMonth = Integer.parseInt(temp);

    output.setMonth(newMonth - 1);
    return output;
   }

   if (position.equalsIgnoreCase("After month")) {

    int currentMonth = input.getMonth() + 1;
    int newMonth = -1;
    String currentMonthStr = String.valueOf(currentMonth);
    while ((currentMonth == newMonth)) {
     randomDigit = random.nextInt(10);
     String randomStr = String.valueOf(randomDigit);
     String temp = currentMonthStr + randomStr;
     newMonth = Integer.parseInt(temp);
    }
    output.setMonth(newMonth - 1);
    return output;
   }

   if (position.equalsIgnoreCase("Before day")) {
    int currentDay = input.getDate();
    String currentDayStr = String.valueOf(currentDay);
    int newDay = -1;


    randomDigit = random.nextInt(9) + 1;
    String randomStr = String.valueOf(randomDigit);
    String temp = randomStr + currentDayStr;
    newDay = Integer.parseInt(temp);


    output.setDate(newDay);
    return output;
   }

   if (position.equalsIgnoreCase("After day")) {
    int currentDay = input.getDate();
    String currentDayStr = String.valueOf(currentDay);
    int newDay = currentDay;

    while ((currentDay == newDay)) {
     randomDigit = random.nextInt(10);
     String randomStr = String.valueOf(randomDigit);
     String temp = currentDayStr + randomStr;
     newDay = Integer.parseInt(temp);

    }
    output.setDate(newDay);
    return output;
   }

   if (position.equalsIgnoreCase("before hour")) {
    randomDigit = random.nextInt(9) + 1;
    hourStr = Integer.toString(randomDigit) + hourStr;
    output.setHours(Integer.parseInt(hourStr));
    return output;
   }

   if (position.equalsIgnoreCase("after hour")) {
    randomDigit = random.nextInt(10);
    hourStr = hourStr + Integer.toString(randomDigit);
    output.setHours(Integer.parseInt(hourStr));
    return output;
   }

   if (position.equalsIgnoreCase("before minute")) {
    randomDigit = random.nextInt(9) + 1;
    minuteStr = Integer.toString(randomDigit) + minuteStr;
    output.setMinutes(Integer.parseInt(minuteStr));
    return output;
   }

   if (position.equalsIgnoreCase("after minute")) {
    randomDigit = random.nextInt(10);
    minuteStr = minuteStr + Integer.toString(randomDigit);
    output.setMinutes(Integer.parseInt(minuteStr));
    return output;
   }

   if (position.equalsIgnoreCase("before second")) {
    randomDigit = random.nextInt(9) + 1;
    secondStr = Integer.toString(randomDigit) + secondStr;
    output.setSeconds(Integer.parseInt(secondStr));
    return output;
   }

   if (position.equalsIgnoreCase("after nano")) {
    randomDigit = random.nextInt(10);
    nanoStr = nanoStr + Integer.toString(randomDigit);
    output.setNanos(Integer.parseInt(nanoStr));
    return output;
   }

   if (position.equalsIgnoreCase("before nano")) {
    randomDigit = random.nextInt(9) + 1;
    nanoStr = Integer.toString(randomDigit) + nanoStr;
    output.setNanos(Integer.parseInt(nanoStr));
    return output;
   }
   return input;
  }
 };

 // Method 10: Duplicate one character on timestamp value
 // Parameters:
 // 	Position: year - month - day - hour - minute - second - nano
 public static final functionTwoParameters m6_duplicateOneDigit = new functionTwoParameters() {

  @Override
  public Timestamp run(Timestamp input, String field) {
   nameOfLastExecutedFunction = "m6_duplicateOneDigit";

   java.sql.Timestamp output = (Timestamp) input.clone();

   // Helper variables
   int year = input.getYear() + 1900;
   int month = input.getMonth() + 1;
   int day = input.getDate();
   int hour = input.getHours();
   int minute = input.getMinutes();
   int second = input.getSeconds();
   int nano = input.getNanos();
   String yearStr = String.valueOf(year);
   String monthStr = String.valueOf(month);
   String dayStr = String.valueOf(day);
   String hourStr = String.valueOf(hour);
   String minuteStr = String.valueOf(minute);
   String secondStr = String.valueOf(second);
   String nanoStr = String.valueOf(nano);

   Random random = new Random();
   int extra = 0;
   int randomPosition = 0;

   if (field.equalsIgnoreCase("year")) {
    randomPosition = random.nextInt(yearStr.length());
    yearStr = yearStr.substring(0, randomPosition) + yearStr.charAt(randomPosition) + yearStr.substring(randomPosition);
    output.setYear(Integer.parseInt(yearStr) - 1900);
   }

   if (field.equalsIgnoreCase("month")) {
    randomPosition = random.nextInt(monthStr.length());
    monthStr = monthStr.substring(0, randomPosition) + monthStr.charAt(randomPosition) + monthStr.substring(randomPosition);
    output.setMonth(Integer.parseInt(monthStr) - 1);
   }

   if (field.equalsIgnoreCase("day")) {
    randomPosition = random.nextInt(dayStr.length());
    dayStr = dayStr.substring(0, randomPosition) + dayStr.charAt(randomPosition) + dayStr.substring(randomPosition);
    output.setDate(Integer.parseInt(dayStr));
   }
   if (field.equalsIgnoreCase("hour")) {
    randomPosition = random.nextInt(hourStr.length());
    hourStr = hourStr.substring(0, randomPosition) + hourStr.charAt(randomPosition) + hourStr.substring(randomPosition);
    output.setHours(Integer.parseInt(hourStr));
   }

   if (field.equalsIgnoreCase("minute")) {
    randomPosition = random.nextInt(minuteStr.length());
    minuteStr = minuteStr.substring(0, randomPosition) + minuteStr.charAt(randomPosition) + minuteStr.substring(randomPosition);
    output.setMinutes(Integer.parseInt(minuteStr));

   }

   if (field.equalsIgnoreCase("second")) {
    randomPosition = random.nextInt(secondStr.length());
    secondStr = secondStr.substring(0, randomPosition) + secondStr.charAt(randomPosition) + secondStr.substring(randomPosition);
    output.setSeconds(Integer.parseInt(secondStr));
   }

   if (field.equalsIgnoreCase("nano")) {
    randomPosition = random.nextInt(nanoStr.length());
    nanoStr = nanoStr.substring(0, randomPosition) + nanoStr.charAt(randomPosition) + nanoStr.substring(randomPosition);
    output.setNanos(Integer.parseInt(nanoStr));
   }

   return output;
  }
 };

 // Method 11: Add 100 years to the timestamp
 public static final functionOneParameter m7_add100years = new functionOneParameter() {

  @Override
  public Timestamp run(Timestamp input) {
   nameOfLastExecutedFunction = "m7_add100years";

   java.sql.Timestamp output = (Timestamp) input.clone();
   output.setYear(input.getYear() + 100);
   return output;
  }
 };

 // Method 12: Subtract 100 years from the timestamp
 public static final functionOneParameter m8_subtract100years = new functionOneParameter() {

  @Override
  public Timestamp run(Timestamp input) {
   nameOfLastExecutedFunction = "m8_subtract100years";

   java.sql.Timestamp output = (Timestamp) input.clone();
   output.setYear(input.getYear() - 100);
   return output;
  }
 };

 // Method 13: Remove first 2 digits of year field
 public static final functionOneParameter m9_removeDigitsFromYear = new functionOneParameter() {

  @Override
  public Timestamp run(Timestamp input) {
   nameOfLastExecutedFunction = "m9_removeDigitsFromYear";

   // year to string and after that substring and choose last 2
   java.sql.Timestamp output = (Timestamp) input.clone();

   int year = input.getYear() + 1900;
   String yearStr = String.valueOf(year);
   yearStr = yearStr.substring(2);
   output.setYear(Integer.parseInt(yearStr) - 1900);
   return output;
  }
 };


}